/*
 * A* implementation for graph navigation. Source: /cite{aigames}
 */

function AStarCurrent(set, values) {
	var min = null;

	for (var i in set) {
		if (!min) { min = i; continue; }
		if (values[set[i]] < values[set[min]]) { min = i; }
	}

	return set[min];
}

function AStarReconstructPath(predecessors, target) {
	var path = [target];

	while (defined(predecessors[target.toString()])) {
		target = predecessors[target.toString()];
		path.unshift(target);
	}

	return path;
}

function addNodes(object, coords, value) {
	var node = graph.findNode(coords);

	for (var i in node.edges) {
		object[i] = value;
	}
}

function AStar(from, to, who, discourageArr) {
	var openSet = [graph.findNode(from).coords];
	var closedSet = {};
	var cameFrom = {};
	var costSoFar = {};
	var estTotalCost = {};
	var discourage = {};

	if (discourageArr) {
		for (var i in discourageArr) {
			discourage[discourageArr[i]] = 1;
		}
	}

	costSoFar[openSet[0].toString()] = 0;
	estTotalCost[openSet[0].toString()] = distance(openSet[0], to);

	while (openSet.length > 0) {
		var current = AStarCurrent(openSet, estTotalCost);

		if (current.isEqual(to)) {
			return AStarReconstructPath(cameFrom, current);
		}

		for (var i in graph.nodes.at(current).edges) {
			var edge = graph.nodes.at(current).edges[i];

			if (edge.water && propulsion[who].terrain === 'land') {
				continue;
			}

			var potentialCost = costSoFar[current.toString()] + edge.dist + discourage[edge.to] ? 100 : 0;

			if (edge.to.toString() in closedSet) {
				if (costSoFar[edge.to.toString()] <= potentialCost) { continue; }
				delete closedSet[edge.to.toString()];
			} else if (arrayContains(openSet, edge.to)) {
				if (costSoFar[edge.to.toString()] <= potentialCost) { continue; }
			}

			cameFrom[edge.to.toString()] = current;
			costSoFar[edge.to.toString()] = potentialCost;
			estTotalCost[edge.to.toString()] = potentialCost + distance(edge.to, to);

			if (!arrayContains(openSet, edge.to)) { openSet.push(edge.to); }
		}

		openSet.splice(openSet.indexOf(current), 1);
		closedSet[current.toString()] = 1;
	}
	return null;
}

function AStarConditioned(from, predicate, who, amount) {
	if (!amount) {
		amount = 1;
	}

	var openSet = [graph.findNode(from).coords];
	var closedSet = {};
	var costSoFar = {};
	var results = [];

	costSoFar[openSet[0].toString()] = 0;

	while (openSet.length > 0) {
		var current = AStarCurrent(openSet, costSoFar);

		if (predicate(graph.nodes.at(current))) {
			results.push(graph.nodes.at(current));
			if (results.length === amount) {
				return results;
			}
		}

		for (var i in graph.nodes.at(current).edges) {
			var edge = graph.nodes.at(current).edges[i];

			if (edge.water && propulsion[who].terrain === 'land') {
				continue;
			}

			var potentialCost = costSoFar[current.toString()] + edge.dist + graph.nodes.at(edge.to).hazard;

			if (edge.to.toString() in closedSet) {
				if (costSoFar[edge.to.toString()] <= potentialCost) { continue; }
				delete closedSet[edge.to.toString()];
			} else if (arrayContains(openSet, edge.to)) {
				if (costSoFar[edge.to.toString()] <= potentialCost) { continue; }
			}

			costSoFar[edge.to.toString()] = potentialCost;

			if (!arrayContains(openSet, edge.to)) { openSet.push(edge.to); }
		}

		openSet.splice(openSet.indexOf(current), 1);
		closedSet[current.toString()] = 1;
	}
	return results;
}