/*
 * Main file.
 */

include("multiplay/skirmish/convnet.js");
include("multiplay/skirmish/predicates.js");
include("/multiplay/skirmish/utils.js");

var bases = [];
var dead = [];
var entrance = {};
var mapTop = '';
var focusedPlayer = -1;
var half = true;
var initialScout = false;
var groupCount = 0;
const FORCE_COUNT = 6;
const SCOUT_DELAY = 90000;
const REINFORCE_RANGE = distBetweenTwoPoints(1, 1, Math.floor(mapWidth / 2), Math.floor(mapHeight / 2)) / 2; // 15;
const HAZARD_DECAY_AMNT = 1;
const HAZARD_DECAY_DELAY = 5000;
const MAX_DIST = 0.1;
const MAX_SQUAD_SIZE = 6;
// const MAX_SUPPORT_SIZE = 3;
const VTOL_THRESHOLD = 0.35;
const SIGHT_RANGE = 10;
const GROUP_DECAY = 180000;
const FORTIFICATION_LIMIT = 4;
const PRESENCE_RADIUS = 7;
const REPAIR_THRESHOLD = 30; // in %

// component and structure libraries
include("/multiplay/skirmish/structures.js");
include("/multiplay/skirmish/bodies.js");
include("/multiplay/skirmish/propulsion.js");
include("/multiplay/skirmish/weapons.js");
include("/multiplay/skirmish/defenses.js");

include("/multiplay/skirmish/mapAnalyzer.js");

var graph = new Graph();

include("/multiplay/skirmish/armyGroups.js");
include("/multiplay/skirmish/AICoreClass.js");
include("/multiplay/skirmish/colonelCore.js");
include("/multiplay/skirmish/micromanager.js");

include("/multiplay/skirmish/router.js");

var designer = new DesignerCore();

include("/multiplay/skirmish/events.js");

function Area(x, y, x2, y2) {
	this.type = AREA;
	this.x = x;
	this.y = y;
	this.x2 = x2;
	this.y2 = y2;
}

function Position(x, y) {
	this.type = POSITION;
	this.x = x;
	this.y = y;
}

function buildersInQ() {
	var f = enumStruct(me, struct['Factory'].id);
	var c = 0;

	for (var i = 0; i < f.length; i++) {
		if (getDroidProduction(f[i]) && getDroidProduction(f[i]).droidType === DROID_CONSTRUCT) { c++; }
	}
	return c;
}

function checkDerrickBuilt(rig) {
	if (rig) {
		graph.nodes.at(new Point(rig.y, rig.x)).type = nodeType.DERRICK_FREE;
	}
}

function produceDroid(fact, name, b, p, w) {
	buildDroid(fact, name, b, p, null, null, w);
}

function produceCyborg(fact, name, w) {
	buildDroid(fact, name, turrets.Weapon[w].body, 'CyborgLegs', null, null, turrents.Weapon[w].id);
}

function getIdleStruc(struc) {
	for (var i = 0; i < countStruct(struc, me); i++) {
		var fact = enumStruct(me, struc)[i];
		if (structureIdle(fact)) {
			return fact;
		}
	}
	return null;
}

function isActivePlayer(id) {
	return (playerData[id].isAI || playerData[id].isHuman) && !dead[id];
}

function checkForPickups() {
	var f = enumFeature(me, 'OilDrum');
	var a = enumFeature(me, 'Crate'); // a.k.a. Artifact

	for (var i = 0; i < f.length; i++) {
		var b = engiCore.getIdleBuilder(engiCore.roamers);
		if (b) { orderDroidObj(b, DORDER_RECOVER, f[i]); return; }
	}
	for (var i = 0; i < a.length; i++) {
		var b = engiCore.getIdleBuilder(engiCore.roamers);
		if (b) { orderDroidObj(b, DORDER_RECOVER, a[i]); return; }
	}
}

function sameEnds(paths) {
	var last = paths[0][paths[0].length - 1];

	for (var i = 0; i < paths.length; i++) {
		if (!last.isEqual(paths[i][paths[i].length - 1])) { return false; }
	}

	return true;
}

function removeEnds(paths) {
	var last = paths[0].pop();
	for (var i = 1; i < paths.length; i++) {
		paths[i].pop();
	}

	return last;
}

function findBaseEntrance(id) {
	var paths = [];

	for (var i = 0; i < bases.length; i++) {
		if (i === id) { continue; }
		paths.push(graph.findPath(bases[i], bases[id], 'Hover'));
	}

	var l = null;
	while (sameEnds(paths)) {
		l = removeEnds(paths);
	}

	return l;
}

var enemyGroups = {};
var enemyBases = {};

Object.defineProperty(enemyGroups, 'droidCount', {
	configurable: true,
	writable: true,
	enumerable: false,
	value: 0
});
Object.defineProperty(enemyGroups, 'vtolCount', {
	configurable: true,
	writable: true,
	enumerable: false,
	value: 0
});
Object.defineProperty(enemyGroups, 'unitCount', {
	configurable: true,
	writable: true,
	enumerable: false,
	value: 0
});
Object.defineProperty(enemyGroups, 'borgCount', {
	configurable: true,
	writable: true,
	enumerable: false,
	value: 0
});

function noticeAroundNodes() {
	var halfindex = Math.floor(graph.allNodes.length / 2);
	var start = half ? 0 : halfindex;
	var limit = half ? halfindex : graph.allNodes.length;

	for (var i = start; i < limit; i++) {
		var n = graph.allNodes[i];
		var derrick = getObject(n.coords.c, n.coords.r);

		if (derrick) {
			if (derrick.stattype === OIL_RESOURCE) {
				if (n.type === nodeType.DERRICK_BUILT || gameTime - n.timeClaimed > 45000) {
					n.type = nodeType.DERRICK_FREE;
				}
			} else if (derrick.stattype === RESOURCE_EXTRACTOR) {
				n.type = nodeType.DERRICK_BUILT;
			}
		}

		var u = enumRange(n.coords.c, n.coords.r, SIGHT_RANGE, ENEMIES);

		if (!u.length) {
			u = enumRange(n.coords.c, n.coords.r, SIGHT_RANGE, me);
		} else {
			var def = u.filter(isDefense);
			n.fortified = def.length;
			if (def.length) {
				n.owner = def[0].player;
			}

			// make sure to catch visible enemies even if further away from node
			var enemies = enumRange(u[0].x, u[0].y, SIGHT_RANGE, ENEMIES).filter(function(e){return e.type === DROID && isCombatant(e);}); //u.filter(isCombatant);

			if (enemies.length) {
				n.lastSeen = gameTime;
				n.owner = enemies[0].player;

				var pos = new Point(enemies[0].y, enemies[0].x);
				var groupInfo = {
					owner: enemies[0].player,
					units: enemies,
					lastSeen: gameTime,
					pos: pos
				}

				if (!Object.keys(enemyGroups).length) {
					enemyGroups[groupCount++] = groupInfo;
					return;
				}

				var spd = (new Template('e', idToName[enemies[0].body], idToName[enemies[0].propulsion], [idToName[enemies[0].weapons[0].id]], 'Weapon')).speed()/128;
				var nearest = { dist: 999999999, group: -1 };

				for (var i in enemyGroups) {
					var dist = distance(pos, enemyGroups[i].pos);
					var time = (gameTime - enemyGroups[i].lastSeen) / 1000;

					if (!time && dist < 6) { return; }

					var reqSpd = dist/time;

					if (reqSpd <= spd) {
						if (nearest.dist > dist) {
							nearest = {dist: dist, group: i};
						}
					}
				}

				if (nearest.group !== -1) {
					enemyGroups[nearest.group] = groupInfo;
				} else {
					enemyGroups[groupCount++] = groupInfo;
				}
			}

			continue;
		}

		if (u.length) {
			var def = u.filter(isDefense);
			n.fortified = def.length;
			n.lastSeen = gameTime;
			n.owner = me;
		}
	}

	enemyGroups['vtolCount'] = 0;
	enemyGroups['borgCount'] = 0;
	enemyGroups['unitCount'] = 0;

	for (var i in enemyGroups) {
		// if (i.match(/Count/)) { continue; }
		enemyGroups['unitCount'] += enemyGroups[i].units.length;
		enemyGroups['vtolCount'] += enemyGroups[i].units.filter(function(u){return u.propulsion === propulsion['VTOL'].id }).length;
		enemyGroups['borgCount'] += enemyGroups[i].units.filter(function(u){return u.droidType === DROID_CYBORG }).length;
	}
	enemyGroups['droidCount'] = enemyGroups['unitCount'] - enemyGroups['vtolCount'] - enemyGroups['borgCount'];

	half = !half;
}

function findOutlyingStrucs() {
	enemyBases = {};

	for (var i in startPositions) {
		if (i == me) { continue; } // soft equality due to i being string

		var s = enumArea(1, 1, mapWidth, mapHeight, i).filter(function(o){
			return o.stattype && o.stattype !== WALL && o.stattype !== DEFENSE && o.stattype !== RESOURCE_EXTRACTOR;
		});

		for (var j in s) {
			var struc = s[j];
			var node = graph.findNode({r: struc.y, c: struc.x});

			if (enemyBases[node.coords]) {
				enemyBases[node.coords].size++;
			} else {
				enemyBases[node.coords] = { owner: i, size: 1, coords: node.coords };
			}
		}

		if (!s.length) {
			for (var base in enemyBases) {
				if (enemyBases[base].owner === i) {
					delete enemyBases[base];
				}
			}
		}
	}
}

function hazardDecay() {
	for (var r = 0; r < graph.nodes.length; r++) {
		for (var c = 0; c < graph.nodes[r].length; c++) {
			if (graph.nodes[r][c] && graph.nodes[r][c].hazard) {
				graph.nodes[r][c].hazard -= HAZARD_DECAY_AMNT;
			}
		}
	}
}

function enemyGroupDecay() {
	for (var i in enemyGroups) {
		if (gameTime - enemyGroups[i].lastSeen > GROUP_DECAY) {
			delete enemyGroups[i];
			delete colonel.threats[i];
			i--;
		}
	}
}