/*
 *	Wrappers around unit API.
 */

function updateGroupPos() {
	for (var j in colonel.sgts) {
		var u = colonel.sgts[j].units.getUnits();

		if (u.length === 0) {
			colonel.sgts[j].units.pos = new Point(bases[me].r, bases[me].c);
			continue;
		}

		var newPos = new Point(u[0].y, u[0].x);
		for (var i = 1; i < u.length; i++) {
			newPos.r = newPos.r + Math.round((u[i].y - newPos.r) / 2 * (1/i));
			newPos.c = newPos.c + Math.round((u[i].x - newPos.c) / 2 * (1/i));
		}

		if (!colonel.sgts[j].units.pos.isEqual(newPos)) {
			colonel.sgts[j].units.lastMoved = gameTime;
		}

		colonel.sgts[j].units.pos.r = newPos.r;
		colonel.sgts[j].units.pos.c = newPos.c;

		colonel.sgts[j].units.graphPos = graph.findNode(colonel.sgts[j].units.pos);

		if (gameTime - colonel.sgts[j].units.lastMoved > 5000 && colonel.sgts[j].status !== state.ENGAGED) {
			colonel.sgts[j].orderDone();
		}

		colonel.sgts[j].units.graphPos.lastSeen = gameTime;
	}
}

const state = {
	IDLE: 0,
	MOVING: 1,
	ENGAGED: 2,
	RETREAT: 3
};

function moveUnitToGroup(unit) {
	if (unit) {
		var a = getDroidsArmy(unit);

		if (a) {
			orderDroidLoc(unit, DORDER_MOVE, a.units.pos.c, a.units.pos.r);
		}
	}
}

// Basic group object
function Group(id) {
	this.units = id; // newGroup();
	this.pos = new Point(bases[me].r, bases[me].c);
	this.graphPos = graph.findNode(bases[me]);
	this.lastMoved = 0;
}

Group.prototype = {
	add: function(unit) {
		groupAdd(this.units, unit);
		queue('moveUnitToGroup', 500, unit);
	},

	size: function() {
		return groupSize(this.units);
	},

	move: function(dest) {
		var u = enumGroup(this.units);
		for (var i = 0; i < u.length; i++) {
			orderDroidLoc(u[i], DORDER_MOVE, dest.c, dest.r);
		}
	},

	retreat: function() {
		this.move(bases[me]);
	},

	repair: function(s) {
		var u = enumGroup(this.units);
		for (var i = 0; i < u.length; i++) {
			orderDroid(u[i], DORDER_RTR);
		}
	},

	health: function() {
		var HP = 0;
		var u = enumGroup(this.units);
		for (var i = 0; i < u.length; i++) {
			HP += u[i].health;
		}
		HP = Math.floor(HP / u.length);

		return HP;
	},

	merge: function(otherArmy) {
		var u = enumGroup(this.units);
		for (var i = 0; i < u.length; i++) {
			groupAdd(otherArmy.units, u[i]);
		}
	},

	getUnits: function() {
		return enumGroup(this.units);
	},

	see: function() {
		var unit = this.getUnits()[0];
		var range = SIGHT_RANGE > Stats.Weapon[idToName[unit.weapons[0].id]].MaxRange/128 ? SIGHT_RANGE : (Stats.Weapon[idToName[unit.weapons[0].id]].MaxRange/128);
		var u = enumRange(this.pos.c, this.pos.r, range, ENEMIES);

		var threat = u.filter(isCombatant).length;
		this.graphPos.hazard += threat;
		for (var n in this.graphPos.edges) {
			graph.nodes.at(this.graphPos.edges[n].to).hazard = Math.floor(threat / 2);
		}

		return u;
	},

	recycle: function() {
		var u = enumGroup(this.units);
			for (var unit in u) {
				orderDroid(u[unit], DORDER_RECYCLE);
			}
		// }
	}
}

// Builder group extension
function BuilderGroup(id) {
	Group.call(this, id);
	this.status = state.IDLE;
}

BuilderGroup.prototype = Object.create(Group.prototype);
BuilderGroup.prototype.constructor = BuilderGroup;

BuilderGroup.prototype.build = function(struc, x, y) {
	var u = this.getUnits();

	if (!this.size()) { return false; }

	var pos;

	if (defined(x) && defined(y)) {
		pos = pickStructLocation(u[0], struc, x, y);
	} else {
		pos = pickStructLocation(u[0], struc, bases[me].c, bases[me].r);
	}

	if (!pos) { return false; }

	var unable = 0;
	u.forEach(function(b) {
		if (!orderDroidBuild(b, DORDER_BUILD, struc, pos.x, pos.y)) {
			unable++;
		}
	});

	return !unable;
}

BuilderGroup.prototype.help = function(struc) {
	var u = enumGroup(this.units);

	if (u.length == 0) { return false; }

	for (var i = 0; i < u.length; i++) {
		orderDroidObj(u[i], DORDER_HELPBUILD, struc);
	}
}

// Attack group extension
function AttackGroup(id) {
	Group.call(this, id);
	this.isVTOL = false;
}

AttackGroup.prototype = Object.create(Group.prototype);
AttackGroup.prototype.constructor = AttackGroup;

AttackGroup.prototype.attackMove = function(dest) {
	var u = enumGroup(this.units);
	for (var i = 0; i < u.length; i++) {
		orderDroidLoc(u[i], DORDER_SCOUT, dest.c, dest.r);
	}
}

AttackGroup.prototype.attack = function(target) {
	var u = enumGroup(this.units);
	for (var i = 0; i < u.length; i++) {
		orderDroidObj(u[i], DORDER_ATTACK, target);
	}
}
AttackGroup.prototype.rearm = function() {
	var u = enumGroup(this.units);
	for (var i = 0; i < u.length; i++) {
		orderDroid(u[i], DORDER_REARM);
	}
}
AttackGroup.prototype.checkAmmo = function() {
	var u = enumGroup(this.units);
	var ammo = 0;
	for (var i = 0; i < u.length; i++) {
		ammo += u[i].armed;
	}

	return ammo / this.size();
}

// Lieutenant group extension
function LtGroup(id, role) {
	this.pos = new Point(bases[me].r, bases[me].c);
	this.graphPos = graph.findNode(bases[me]);
	this.units = id;
  this.support = {
		repair: new Group(id+100),
		borgRepair: new Group(id+200),
		anti_air: new AttackGroup(id+300)
	};

	if (mapTop.match(/land/)) {
		if (role === 'main') {
			this.support.borg = new AttackGroup(id+400);
		} else {
			this.support.main = new AttackGroup(id+400);
		}
	}
}

LtGroup.prototype.add = function(unit) {
	if (this.support[unit.name]) {
		groupAdd(this.support[unit.name].units, unit);
	} else {
		groupAdd(this.units, unit);
	}
	queue('moveUnitToGroup', 500, unit);
}

LtGroup.prototype.size = function() {
	var size = 0;
	size += groupSize(this.units);

	for (var i in this.support) {
		if (i.match(/droid|borg$/)) {
			size += groupSize(this.support[i].units);
		} else {
			if (size && groupSize(this.support[i].units)) {
				size++; // repairs and anti_air do affect strength, however negligently
			}
		}
	}

	return size;
}

LtGroup.prototype.mainSize = function() {
	return groupSize(this.units);
}

LtGroup.prototype.move = function(dest) {
	var u = enumGroup(this.units);

	for (var i in this.support) {
		u = u.concat(enumGroup(this.support[i].units))
	}
	for (var i = 0; i < u.length; i++) {
		orderDroidLoc(u[i], DORDER_MOVE, dest.c, dest.r);
	}
}

LtGroup.prototype.retreat = function() {
	this.move(bases[me]);
}

LtGroup.prototype.getUnits = function() {
	var u = enumGroup(this.units);

	for (var i in this.support) {
		u = u.concat(enumGroup(this.support[i].units))
	}
	return u;
}

LtGroup.prototype.attackMove = function(dest) {
	//log("sending in attack move");
	var u = enumGroup(this.units);

	for (var i in this.support) {
		u = u.concat(enumGroup(this.support[i].units))
	}

	for (var i = 0; i < u.length; i++) {
		orderDroidLoc(u[i], DORDER_SCOUT, dest.c, dest.r);
	}
}

LtGroup.prototype.attack = function(target) {
	//log("sending in attack");
	var u = enumGroup(this.units);

	for (var i in this.support) {
		u = u.concat(enumGroup(this.support[i].units))
	}

	for (var i = 0; i < u.length; i++) {
		orderDroidObj(u[i], DORDER_ATTACK, target);
	}
}

LtGroup.prototype.see = function() {
	var u = enumRange(this.pos.c, this.pos.r, 12, ENEMIES);

	var threat = u.filter(isCombatant).length;
	this.graphPos.hazard += threat;
	for (var n in this.graphPos.edges) {
		graph.nodes.at(this.graphPos.edges[n].to).hazard = Math.floor(threat / 2);
	}

	return u;
}

function getDroidsArmy(droid) {
	if (!colonel.sgts[droid.group]) { return null; }

	return colonel.sgts[droid.group];
}