/*
 *	Definition of colonel core, responsible for tactical control.
 */

include("/multiplay/skirmish/microCore.js");
include("/multiplay/skirmish/sergeantCore.js");
include("/multiplay/skirmish/airSergeantCore.js");

function ColonelCore() {
	this.sgts = [];
	this.targets = {};
	this.threats = {};

	this.assignDroid = function(droid) {
		for (var i in this.sgts) {
			if (this.sgts[i].role === droid.name && this.sgts[i].isUnderstrength() && this.sgts[i].prop === idToName[droid.propulsion]) {
				this.sgts[i].units.add(droid);
				return;
			}
		}

		var nsg;

		console('adding ' + droid.name + ' sgt due to overproduction');

		if (droid.propulsion === propulsion['VTOL'].id) {
			nsg = new AirSergeantCore(droid.name, this.sgts.length);
		} else {
			nsg = new ArmySergeantCore(droid.name, this.sgts.length, idToName[droid.propulsion]);
		}
		this.sgts.push(nsg);
		nsg.units.add(droid);
	}

	this.sendAssistance = function(group, allies, enemy) {
		var support = 0;
		var toSend = [];

		for (var i = 0; i < this.sgts.length; i++) {
			if (i === group.units || this.sgts[i].isEmpty()) { continue; }
			if (distance(group.pos, this.sgts[i].units.pos) < REINFORCE_RANGE) {
				toSend[i] = 1;
				support += this.sgts[i].units.size();
			}
			if (this.sgts[i].role === 'gunship' && this.sgts[i].status === state.IDLE) {
				toSend[i] = 1;
				support += this.sgts[i].units.size();
			}
		}

		if (allies + support > enemy) {
			for (var i in toSend) {
				this.sgts[i].order('attack', group.pos);
			}
			return true;
		}

		return false;
	}

	this.changeSetup = function() {
		this.sgts.push(new ArmySergeantCore('droid_pers', this.sgts.length, 'Wheels'));
	}

	this.commitToTarget = function(sgt, task, target) {
		if (!this.targets[target]) {
			this.targets[target] = { coords: target, task: task, force: {} };
		}

		this.targets[target].force[sgt.units.units] = sgt.units.size();
	}

	/*
	 *	Plans out actions for main sergeants.
	 */
	this.mainPlan = function(sgts) {
		var targets = this.sortByValue(this.threats);
		var target = targets[0];
		var potential = {};
		var commited = {};

		potential[target] = { sgts: [], units: 0 };
		for (var j = 0; j < sgts.length; j++) {
			var sgt = sgts[j];
			var d = graph.nearestNode(bases[me], enemyDerrick, designer.templates[sgt.role].prop, 5);

			switch (mainCore.microPlan) {
				case 'skirmish': {
					var safest = null;

					for (var i = 0; i < d.length; i++) {
						if (mapTop !== 'land' && !propulsionCanReach(propulsion[designer.templates[sgt.role].prop].id, sgt.units.pos.c, sgt.units.pos.r, d[i].coords.c, d[i].coords.r)) {
							continue;
						}
						if (!safest || (d[i].hazard < safest.hazard && !commited[safest.coords])) {
							safest = d[i];
						}
					}

					if (safest) { // harass
						var coords = safest.coords;
						if (!potential[coords]) {
							potential[coords] = { sgts: [sgt.units.units], units: sgt.units.size() };
						} else {
							potential[coords].sgts.push(sgt.units.units);
							potential[coords].units += sgt.units.size();
						}

						if (safest.hazard < potential[coords].units) {
							// commit
							commited[coords] = { coords: coords, task: 'attack', force: potential[coords] };
							delete potential[coords];
						}
						continue;
					}

					var busy = false;
					for (var i in targets) {
						if (enemyGroups[i].units.length < sgt.units.size() / 2) {
							commited[coords] = { coords: enemyGroups[i].pos, task: 'attack', force: { sgts: [sgt.units.units], units: sgt.units.size() } };
							busy = true;
							break;
						}
					}

					if (!busy) { // scout
						var fogged = graph.nearestNode(sgt.units.graphPos.coords, unscoutedNode, designer.templates[sgt.role].prop)[0];

						if (fogged) {
							commited[fogged.coords] = { coords: fogged.coords, task: 'scout', force: { sgts: [sgt.units.units] } };
						}
					}
					break;
				}
				case 'attack': {
					if (defined(target) && enemyGroups[target].units.length < potential[target].units) {
						potential[target].sgts.push(sgt.units.units);
						potential[target].units += sgt.units.size();

						if (enemyGroups[target].units.length < potential[target].units) {
							commited[target] = { coords: target, task: 'attack', force: potential[target] };
							delete potential[target];
							targets.shift();
							target = targets[0];
						}

						continue;
					}

					if (focusedPlayer >= 0) {
						if (!commited[bases[focusedPlayer]]) {
							commited[bases[focusedPlayer]] = { coords: bases[focusedPlayer], task: 'attack', force: { sgts: [sgt.units.units], units: sgt.units.size() } };
						} else {
							commited[bases[focusedPlayer]].force.sgts.push(sgt.units.units);
							commited[bases[focusedPlayer]].units += sgt.units.size();
						}
					} else {
						var fogged = graph.nearestNode(sgt.units.graphPos.coords, unscoutedNode, designer.templates[sgt.role].prop)[0];

						if (fogged && !this.targets[fogged.coords]) {
							if (!commited[fogged.coords]) {
								commited[fogged.coords] = { coords: fogged.coords, task: 'scout', force: { sgts: [sgt.units.units], units: sgt.units.size() } };
							} else if (commited[fogged.coords].task !== 'scout') {
								commited[base].force.sgts.push(sgt.units.units);
								commited[bases[focusedPlayer]].units += sgt.units.size();
							}
						}
					}
				}
				case 'defend': {
					if (target) {
						var path = graph.findPath(enemyGroups[target].pos, bases[me], designer.templates[sgt.role].prop);
						var nearestUnfortified = null;
						var busy = false;

						for (var i = 0; i < path.length; i++) {
							var n = graph.nodes.at(path[i]);
							if (n.isChokepoint) {
								if (n.fortified) {
									if (!commited[n.coords]) {
										commited[n.coords] = { coords: n.coords, task: 'hold', force: { sgts: [sgt.units.units], units: sgt.units.size() } };
									} else {
										commited[n.coords].force.sgts.push(sgt.units.units);
										commited[n.coords].units += sgt.units.size();
									}

									busy = true;
									break;
								}
								if (n.fortified < FORTIFICATION_LIMIT) {
									nearestUnfortified = n;
								}
							}
						}

						if (nearestUnfortified) {
							sendRequest('engi', 'defense', nearestUnfortified.coords);
						}

						if (busy) { continue; }
					}

					if (entrance[me]) {
						this.commitToTarget(sgt, 'hold', entrance[me]);
						sgt.order('hold', entrance[me]);
					} else {
						this.commitToTarget(sgt, 'hold', bases[me]);
						sgt.order('hold', bases[me]);
					}
				}
			}
		}

		for (var tgt in commited) {
			var groups = commited[tgt].force.sgts;

			for (var i = 0; i < groups.length; i++) {
				this.commitToTarget(this.sgts[groups[i]], commited[tgt].task, commited[tgt].coords);
				this.sgts[groups[i]].order(commited[tgt].task, commited[tgt].coords);
			}
		}

		for (var tgt in potential) {
			var groups = potential[tgt].sgts;
			for (var i = 0; i < groups.length; i++) {
				if (entrance[me]) {
					this.commitToTarget(this.sgts[groups[i]], 'hold', entrance[me]);
					this.sgts[groups[i]].order('hold', entrance[me]);
				} else {
					this.commitToTarget(this.sgts[groups[i]], 'hold', bases[me]);
					this.sgts[groups[i]].order('hold', bases[me]);
				}
			}
		}
	}

	/*
	 *	Plans out actions for VTOL sergeants.
	 */
	this.vtolPlan = function(sgts) {
		var targets = this.sortByValue(this.threats);
		var target = targets[0];
		var potential = {};
		var commited = {};

		potential[target] = { sgts: [], units: 0 };
		for (var j = 0; j < sgts.length; j++) {
			var sgt = sgts[j];
			var d = graph.nearestNode(bases[me], enemyDerrick, designer.templates[sgt.role].prop, 5);

			switch (mainCore.microPlan) {
				case 'defend':
				case 'skirmish': {
					var safest = null;

					for (var i = 0; i < d.length; i++) {
						if (mapTop !== 'land' && !propulsionCanReach(propulsion[designer.templates[sgt.role].prop].id, sgt.units.pos.c, sgt.units.pos.r, d[i].coords.c, d[i].coords.r)) {
							continue;
						}
						if (!safest || (d[i].hazard < safest.hazard && !commited[safest.coords])) {
							safest = d[i];
						}
					}

					if (safest) { // harass
						var coords = safest.coords;
						if (!potential[coords]) {
							potential[coords] = { sgts: [sgt.units.units], units: sgt.units.size() };
						} else {
							potential[coords].sgts.push(sgt.units.units);
							potential[coords].units += sgt.units.size();
						}

						if (safest.hazard < potential[coords].units) {
							// commit
							commited[coords] = { coords: coords, task: 'attack', force: potential[coords] };
							delete potential[coords];
						}
						continue;
					}

					var busy = false;
					for (var i in targets) {
						if (enemyGroups[i].units.length < sgt.units.size() / 2) {
							commited[coords] = { coords: enemyGroups[i].pos, task: 'attack', force: { sgts: [sgt.units.units], units: sgt.units.size() } };
							busy = true;
							break;
						}
					}

					if (!busy) { // scout
						var fogged = graph.nearestNode(sgt.units.graphPos.coords, unscoutedNode, designer.templates[sgt.role].prop)[0];

						if (fogged) {
							commited[fogged.coords] = { coords: fogged.coords, task: 'scout', force: { sgts: [sgt.units.units] } };
						}
					}
					break;
				}
				case 'attack': {
					if (defined(target) && enemyGroups[target].units.length < potential[target].units) {
						potential[target].sgts.push(sgt.units.units);
						potential[target].units += sgt.units.size();

						if (enemyGroups[target].units.length < potential[target].units) {
							commited[target] = { coords: target, task: 'attack', force: potential[target] };
							delete potential[target];
							targets.shift();
							target = targets[0];
						}

						continue;
					}

					if (focusedPlayer >= 0) {
						if (!commited[bases[focusedPlayer]]) {
							commited[bases[focusedPlayer]] = { coords: bases[focusedPlayer], task: 'attack', force: { sgts: [sgt.units.units], units: sgt.units.size() } };
						} else {
							commited[bases[focusedPlayer]].force.sgts.push(sgt.units.units);
							commited[bases[focusedPlayer]].units += sgt.units.size();
						}
					} else {
						var fogged = graph.nearestNode(sgt.units.graphPos.coords, unscoutedNode, designer.templates[sgt.role].prop)[0];

						if (fogged && !this.targets[fogged.coords]) {
							if (!commited[fogged.coords]) {
								commited[fogged.coords] = { coords: fogged.coords, task: 'scout', force: { sgts: [sgt.units.units], units: sgt.units.size() } };
							} else if (commited[fogged.coords].task !== 'scout') {
								commited[base].force.sgts.push(sgt.units.units);
								commited[bases[focusedPlayer]].units += sgt.units.size();
							}
						}
					}
				}
			}
		}
	}

	this.plan = function() {
		if (!mapTop.match(/land/)) {
			var crossWater = true;
			for (var i in enemyGroups) {
				if (propulsionCanReach(propulsion['Wheels'].id, bases[me].c, bases[me].r, enemyGroups[i].pos.c, enemyGroups[i].pos.r)) {
					crossWater = false;
					break;
				}
			}

			if (crossWater) {
				for (var i in enemyBases) {
					if (propulsionCanReach(propulsion['Wheels'].id, bases[me].c, bases[me].r, enemyBases[i].coords.c, enemyBases[i].coords.r)) {
						crossWater = false;
						break;
					}
				}
			}

			if (crossWater) {
				for (var i in bases) {
					if (i == me) { continue; }
					if (isActivePlayer(i) && propulsionCanReach(propulsion['Wheels'].id, bases[me].c, bases[me].r, bases[i].c, bases[i].r)) {
						crossWater = false;
						break;
					}
				}
			}

			if (crossWater) {
				for (var sgt in this.sgts) {
					if (propulsion[this.sgts[sgt].prop].terrain === 'land' && propulsion[designer.templates[this.sgts[sgt].role].prop].terrain !== 'land') {
						this.sgts[sgt].order('disband');
						this.sgts[sgt].status = state.MOVING;
					} else {
						sendRequest('res', 'Hover');
					}
				}

				designer.preferWater = true;
			}

			sendRequest('res', 'Hover');
		}

		this.mainPlan(this.sgts.filter(function(s){return s.role.match(/droid|borg/) && !s.isEmpty() && s.status === state.IDLE;}));
		this.vtolPlan(this.sgts.filter(function(s){return s.role === 'gunship' && !s.isEmpty() && s.status === state.IDLE;}));

		if (mainCore.currentPlan === 'turtle') {
			graph.allNodes.filter(function(n){
				return n.type === nodeType.DERRICK_BUILT && n.owner === me && n.fortified < FORTIFICATION_LIMIT;
			}).forEach(function(n){
				if (engiCore.checkpoints[n.coords]) {
					engiCore.checkpoints[n.coords] += mainCore.increments['turtle'];
				} else {
					engiCore.checkpoints[n.coords] = mainCore.increments['turtle'];
				}
			});

			if (graph.nodes.at(entrance[me]).fortified < FORTIFICATION_LIMIT) {
				sendRequest('engi', 'defense', entrance[me]);
			}

			if (graph.nodes.at(bases[me]).fortified < FORTIFICATION_LIMIT) {
				sendRequest('engi', 'defense', bases[me]);
			}
		}
	}

	this.abortPlan = function(target) {
		if (this.targets[target]) {
			for (var i in this.targets[target].force) {
				this.sgts[i].retreat();
			}
		}
	}

	this.assignRole = function() {
		var wpn = null;
		var type = null;
		if (enemyGroups['droidCount'] >= enemyGroups['borgCount']) {
			wpn = 'tank';
		} else {
			wpn = 'pers';
		}
		var enemyMods = this.sortByValue(enemyDesigns.mods);
		// assuming mains all have same prop/body

		if (enemyMods.length) {
			if (designer.preferWater || dmgModifiers[enemyMods[0]]['Cyborg Propulsion'] > dmgModifiers[enemyMods[0]][designer.templates['droid_pers']]) {
				type = 'droid';
			} else {
				type = 'borg';
			}
		} else {
			type = 'droid';
		}

		return [type, wpn];
	}

	this.evalThreats = function() {
		if (!designer.templates['droid_pers'] || !designer.templates['borg_pers'] || !Object.keys(enemyGroups).length) { return; }

		for (var i in enemyGroups) {
			this.threats[i] = enemyGroups[i].units.length / distance(bases[me], enemyGroups[i].pos);
		}

		var sorted = this.sortByValue(this.threats);
		var cyborgs = enumDroid(me, DROID_CYBORG);
		var armedDroids = enumDroid(me, DROID_WEAPON);
		var mainsgts = this.sgts.filter(function(s){return s.role.match(/droid|borg/);});
		// var raidsgts = this.sgts.filter(function(s){return s.role === 'raider';});
		var mainUnits = 0;

		for (var i in mainsgts) {
			mainUnits += mainsgts[i].units.size();
		}

		var queuedUnits = mainsgts.length * MAX_SQUAD_SIZE - mainUnits;

		if (enemyGroups['borgCount'] > enemyGroups['droidCount'] * 1.2) {
			if (prodCore.increments['pers'] < 10) {
				prodCore.increments['pers']++;
			}

			if (prodCore.increments['tank'] > 1) {
				prodCore.increments['tank']--;
			}
		}
		if (enemyGroups['droidCount'] > enemyGroups['borgCount'] * 1.2) {
			if (prodCore.increments['tank'] < 10) {
				prodCore.increments['tank']++;
			}

			if (prodCore.increments['pers'] > 1) {
				prodCore.increments['pers']--;
			}
		}

		if (enemyGroups['unitCount'] > (armedDroids.length + cyborgs.length) + queuedUnits) {
			var suggestedRole = this.assignRole();
			var type = suggestedRole.join('_');

			console('adding ' + type + ' because ' + enemyGroups['unitCount'] + ' > ' + (armedDroids.length + cyborgs.length + queuedUnits));

			if (designer.templates[type]) {
				this.sgts.push(new ArmySergeantCore(type, this.sgts.length, designer.templates[type].prop));
			} else {
				this.sgts.push(new ArmySergeantCore('droid_pers', this.sgts.length, designer.templates['droid_pers'].prop));
			}
		}

		if (enemyGroups[sorted[0]].units.length/2 > armedDroids.length + cyborgs.length) {
			var path = graph.findPath(enemyGroups[sorted[0]].pos, bases[me], 'Hover');

			for (var i = path.length - 1; i >= 0; i--) {
				var n = graph.nodes.at(path[i]);
				if ((n.type === nodeType.STARTPOS || n.isChokepoint) && n.fortified < 4) {
					sendRequest('engi', 'defense', path[i]);
					break;
				}
			}
		}

		if (enemyGroups['vtolCount'] / enemyGroups['unitCount'] >= VTOL_THRESHOLD) {
			// var needAA = this.sgts.filter(function(s){return s.role.match(/droid|borg$/);});
			// for (var i = 0; i < needAA; i++) {
			// 		this.sgts[i].needAA = true;
			// }
			// var fighters = this.sgts.filter(function(s){return s.role === 'fighter';}).length;

			// if (fighters * MAX_SQUAD_SIZE < enemyGroups['vtolCount']) {
			// 	this.sgts.push(new AirSergeantCore('fighter', this.sgts.length));
			// }
		} else {
			// var needAA = this.sgts.filter(function(s){return s.role.match(/droid|borg$/);});
			// for (var i = 0; i < needAA; i++) {
			// 		this.sgts[i].needAA = false;
			// }
			var gs = this.sgts.filter(function(s){return s.role === 'gunship';}).length;

			if (!gs) {
				this.sgts.push(new AirSergeantCore('gunship', this.sgts.length));
			}
		}

		if (focusedPlayer >= 0) {
			var pathToBase = graph.findPath(bases[me], bases[focusedPlayer], 'Hover');

			for (var i in pathToBase) {
				var node = graph.nodes.at(pathToBase[i]);
				if (node.owner !== me && node.fortified > this.sgts.filter(function(s){return s.role === 'droid_strc';}).length * MAX_SQUAD_SIZE) {
					this.sgts.push(new ArmySergeantCore('droid_strc', this.sgts.length, designer.templates['droid_strc'] ? designer.templates['droid_strc'].prop : designer.templates['droid_pers'].prop));
				}
			}
		}
	}

	this.process = function() {
		sendRequest('engi', 'defense', entrance[me]);

		for (var i in this.targets) {
			if (!Object.keys(this.targets[i]).length) {
				delete this.targets[i];
			}
		}

		this.evalThreats();
		this.plan();
	}
}

ColonelCore.prototype = Object.create(MacroCore.prototype);
ColonelCore.prototype.contructor = ColonelCore;