var expectedOpponent = { type: "Wheels", gun: "Machinegun", body: "Viper" };

function Template(name, b, p, w, turretClass) {
	this.name = name;
	this.body = b;
	this.prop = p;
	this.weapons = w; // HAS TO BE ARRAY
	this.wpnType = turretClass;

	this.cost = function() {
		var cost = Stats.Body[this.body].BuildPower + Stats.Propulsion[this.prop].BuildPower;

		for (var i in this.weapons) {
			cost += Stats[this.wpnType][this.weapons[i]].BuildPower;
		}
		return cost;
	}

	this.HP = function() {
		var HP = Upgrades[me].Body[this.body].HitPoints + Stats.Propulsion[this.prop].HitpointPctOfBody / 100 * Upgrades[me].Body[this.body].HitPoints;

		for (var i in this.weapons) {
			HP += Stats[this.wpnType][this.weapons[i]].HitPoints;
		}

		// HP *= Upgrades[me].Body[b].HitPointPct / 100;

		return HP;
	}

	this.armour = function() {
		return Upgrades[me].Body[this.body].Armour;
	}

	this.thermal = function() {
		return Upgrades[me].Body[this.body].Thermal;
	}

	this.productionPoints = function() {
		var PP = Stats.Body[this.body].BuildTime + Stats.Propulsion[this.prop].BuildTime + (Stats.Body[this.body].BuildTime - 100)/100 * Stats.Propulsion[this.prop].BuildTime;

		for (var i in this.weapons) {
			PP += Stats[this.wpnType][this.weapons[i]].BuildTime;
		}

		return PP;
	}

	this.damage = function(targetProp, targetBody) {
		var damage = 0;

		for (var i in this.weapons) {
			var wpnDmg = Upgrades[me].Weapon[this.weapons[i]].Damage;
			var dmgMod = turrets.Weapon[this.weapons[i]].modifiers[targetProp];

			var armorType = (Stats.Weapon[this.weapons[i]].ImpactType === "HEAT" ? "Thermal" : "Armour");
			var dmgAfterArmor = targetBody ? wpnDmg * dmgMod - Stats.Body[targetBody][armorType] : 1;
			var minDmg = wpnDmg * dmgMod * Stats.Weapon[this.weapons[i]].MinimumDamage / 100;
			var realDmg = ((dmgAfterArmor > minDmg) ? dmgAfterArmor : minDmg);

			damage += realDmg * Upgrades[me].Weapon[this.weapons[i]].HitChance / 100 / (Upgrades[me].Weapon[this.weapons[i]].FirePause / 1000);
		}

		return damage;
	}

	this.speed = function() {
		var penalty = 1;
		var terrMult = this.prop === 'VTOL' ? 2.5 : 1;

		if (Stats.Body[this.body].Size && this.prop === 'VTOL') {
			penalty = Stats.Body[this.body].Size === 1 ? 0.75 : 0.25;
		}

		var maxSpd = Stats.Propulsion[this.prop].MaxSpeed;
		var weight = Stats.Body[this.body].Weight + Stats.Propulsion[this.prop].Weight * Stats.Body[this.body].Weight/100;

		for (var i in this.weapons) {
			weight += Stats[this.wpnType][this.weapons[i]].Weight;
		}

		var baseSpd = Upgrades[me].Body[this.body].Power * maxSpd * terrMult / weight;
		var spd = baseSpd * penalty;

		return maxSpd > spd ? spd : maxSpd;
	}
}

function DesignerCore() {
	// this.wantedComponents = { wpn: null, prop: null, body: null, defense: null };
	this.templates = {
		raider: null,
		droid_pers: null,
		droid_tank: null,
		droid_strc: null,
		bomber: null,
		gunship: null,
		borg_pers: null,
		borg_tank: null,
		borg_strc: null,
		anti_air: null,
		fighter: null,
		// sensor: null,
		builder: new Template('builder', 'Viper', 'Wheels', ['Truck'], 'Construct'),
		repair: null,
		borgBuilder: null,
		borgRepair: null
	};
	this.availableBodies = [];
	this.availablePropulsions = [];
	this.availableWeapons = {
		droid_pers: [],
		droid_tank: [],
		droid_strc: [],
		gunship: [],
		bomber: [],
		fighter: [], // containment for AA only weapons
		anti_air: [], // containment for AA only weapons
		borg_pers: [],
		borg_tank: [],
		borg_strc: []
	};
	this.enemy = new Template('e', 'Viper', 'Wheels', ['Machinegun'], 'Weapon');
	this.wpnFocus = {};

	this.refocus = function() {
		var types = [];

		for (var i = 0; i < 5; i++) {
			types.push(Stats.Weapon[wpnCats['droid_pers'][i]].ImpactClass);
		}

		this.wpnFocus['pers'] = types[randomInt(types.length)];
		types = [];

		for (var i = 0; i < 5; i++) {
			types.push(Stats.Weapon[wpnCats['droid_tank'][i]].ImpactClass);
		}

		this.wpnFocus['tank'] = types[randomInt(types.length)];
		types = [];

		for (var i = 0; i < 5; i++) {
			types.push(Stats.Weapon[wpnCats['droid_strc'][i]].ImpactClass);
		}

		this.wpnFocus['strc'] = types[randomInt(types.length)];
	}

	this.designDroid = function(type) {
		var abort = false;
		if (!this.availableBodies.length) {
			if (resCore) {
				for (var i in body) {
					if (!componentAvailable(body[i].id)) {
						sendRequest('res', i);
						break;
					}
				}
			}
			abort = true;
		}
		if (!this.availablePropulsions.length) {
			if (resCore) {
				for (var i in propulsion) {
					if (!componentAvailable(propulsion[i].id)) {
						sendRequest('res', i);
						break;
					}
				}
			}
			abort = true;
		}
		if (!this.availableWeapons[type].length) {
			if (resCore) {
				sendRequest('res', wpnCats[type][0]);
			}
			abort = true;
		}

		if (!countStruct(struct['Command Center'].id , me) || enumStruct(me, struct['Command Center'].id)[0].status === BEING_BUILT) {
			sendRequest('engi', 'Command Center');
			abort = true;
		}

		if (abort) {
			return;
		}

		var template = null;
		if (this.templates[type]) {
			template = new Template(type, this.templates[type].body, this.templates[type].prop, this.templates[type].weapons, 'Weapon');
		} else {
			template = new Template(type, this.availableBodies[0], this.availablePropulsions[0], [this.availableWeapons[type][0]], 'Weapon');
			this.templates[type] = new Template(type, this.availableBodies[0], this.availablePropulsions[0], [this.availableWeapons[type][0]], 'Weapon');
		}

		if (type === 'raider') { // speed over health
			// pick prop
			for (var i in this.availablePropulsions) {
				var p = this.availablePropulsions[i];
				template.prop = p;

				if ((this.preferWater && propulsion[p].terrain !== 'land') || template.speed() > this.templates[type].speed()) {
					this.templates[type] = template;
				}
			}

			// pick wpns
			for (var i in this.availableWeapons[type]) {
				var w = this.availableWeapons[type][i];
				template.weapons = [w];

				if (template.damage('strucMedium') > this.templates[type].damage('strucMedium') && template.speed() >= this.templates[type].speed()) {
					this.templates[type] = template;
				}
			}

			// pick body
			for (var i in this.availableBodies) {
				var b = this.availableBodies[i];
				template.body = b;

				if (template.speed() >= this.templates[type].speed()) {
					this.templates[type] = template;
				}
			}

		} else { // health over speed
			// pick body
			for (var i in this.availableBodies) {
				var b = this.availableBodies[i];
				template.body = b;

				var armorType = (Stats.Weapon[this.enemy.weapons[0]].ImpactType === "HEAT" ? "thermal" : "armour");
				if (template.HP() > this.templates[type].HP() && template[armorType]() > this.templates[type][armorType]()) {
					this.templates[type] = template;
				}
			}

			// pick prop
			for (var i in this.availablePropulsions) {
				var p = this.availablePropulsions[i];
				template.prop = p;

				if ((this.preferWater && propulsion[p].terrain !== 'land') || this.enemy.damage(template.prop, template.body) < this.enemy.damage(this.templates[type].prop, this.templates[type].body)) {
					this.templates[type] = template;
				}
			}

			// pick gun
			for (var i in this.availableWeapons[type]) {
				var w = this.availableWeapons[type][i];
				template.weapons = [w];

				if (template.damage(this.enemy.prop, this.enemy.body) > this.templates[type].damage(this.enemy.prop, this.enemy.body)) {
					for (var i = 1; i < Stats.Body[template.body].WeaponSlots; i++) {
						template.weapons.push(w);
					}
					this.templates[type] = template;
				}
			}
		}

		this.requestResearch(type);
	}

	this.designVtol = function(type) {
		if (!componentAvailable(propulsion['VTOL'].id)) {
			return;
		}
		if (!this.availableBodies.length || !this.availableWeapons[type].length) {
			if (resCore) {
				sendRequest('res', wpnCats[type][0]);
			}
			return;
		}
		if (!countStruct(struct['Command Center'].id , me) || enumStruct(me, struct['Command Center'].id)[0].status === BEING_BUILT) {
			sendRequest('engi', 'Command Center');
			return;
		}
		var template = null;

		if (this.templates[type]) {
			template = new Template(type, this.templates[type].body, this.templates[type].prop, this.templates[type].weapons, 'Weapon');
		} else {
			var wpn = this.availableWeapons[type][0];

			template = new Template(type, this.availableBodies[0], 'VTOL', [wpn], 'Weapon');
			this.templates[type] = new Template(type, this.availableBodies[0], 'VTOL', [wpn], 'Weapon');
		}

		if (type === 'bomber') {
			// pick wpns
			for (var i in this.availableWeapons[type]) {
				var w = this.availableWeapons[type][i];
				template.weapons = [w];

				if (template.damage('strucMedium') > this.templates[type].damage('strucMedium') && template.speed() >= this.templates[type].speed()) {
					this.templates[type] = template;
				}
			}

			// pick body
			for (var i in this.availableBodies) {
				var b = this.availableBodies[i];
				template.body = b;

				if (template.speed() >= this.templates[type].speed()) {
					this.templates[type] = template;
				}
			}
		} else {
			for (var i in this.availableBodies) {
				var b = this.availableBodies[i];
				template.body = b;

				var armorType = (Stats.Weapon[this.enemy.weapons[0]].ImpactType === "HEAT" ? "thermal" : "armour");
				if (template.HP() > this.templates[type].HP() && template[armorType]() > this.templates[type][armorType]()) {
					this.templates[type] = template;
				}
			}

			for (var i in this.availableWeapons[type]) {
				var w = this.availableWeapons[type][i];
				template.weapons = [w];

				if (template.damage(this.enemy.prop, this.enemy.body) > this.templates[type].damage(this.enemy.prop, this.enemy.body)) {
					for (var i = 1; i < Stats.Body[template.body].WeaponSlots; i++) {
						template.weapons.push(w);
					}
					this.templates[type] = template;
				}
			}
		}

		this.requestResearch(type);
	}

	this.designBorg = function(type) {
		if (!componentAvailable(cyborgPropulsion['Cyborg Propulsion'].id) || !mapTop.match(/land/)) {
			return;
		}

		if (type === 'borgBuilder') {
			if (!this.templates[type] && componentAvailable(turrets.Construct["*Combat Engineer*"].id)) {
				this.templates[type] = new Template(type, turrets.Construct["*Combat Engineer*"].body, 'Cyborg Propulsion', ["*Combat Engineer*"], 'Construct');
			}
			return;
		}

		if (type === 'borgRepair') {
			if (!this.templates[type] && componentAvailable(turrets.Repair["*Cyborg Mechanic*"].id)) {
				this.templates[type] = new Template(type, turrets.Repair["*Cyborg Mechanic*"].body, 'Cyborg Propulsion', ["*Cyborg Mechanic*"], 'Repair');
			}
			return;
		}

		if (!this.availableWeapons[type].length) {
			if (resCore) {
				sendRequest('res', wpnCats[type][0]);
			}
			return;
		}

		var template = null;

		if (this.templates[type]) {
			template = new Template(type, this.templates[type].body, this.templates[type].prop, this.templates[type].weapons, 'Weapon');
		} else {
			var wpn = this.availableWeapons[type][0];
			template = new Template(type, turrets.Weapon[wpn].body, 'Cyborg Propulsion', [wpn], 'Weapon');
			this.templates[type] = new Template(type, turrets.Weapon[wpn].body, 'Cyborg Propulsion', [wpn], 'Weapon');
		}

		for (var i = 1; i < this.availableWeapons[type].length; i++) {
			var w = this.availableWeapons[type][i];
			template.weapons = [w];
			template.body = turrets.Weapon[w].body;

			if (template.damage(this.enemy.prop, this.enemy.body) > this.templates[type].damage(this.enemy.prop, this.enemy.body)) {
				this.templates[type] = template;
			}
		}
	}

	this.designBuilder = function() {
		var template = null;
		var type = 'builder';

		if (!countStruct(struct['Command Center'].id , me) || enumStruct(me, struct['Command Center'].id)[0].status === BEING_BUILT) {
			sendRequest('engi', 'Command Center');
			return;
		}

		if (this.templates[type]) {
			template = new Template(type, this.templates[type].body, this.templates[type].prop, this.templates[type].weapons, 'Construct');
		} else {
			template = new Template("builder", "Viper", "Wheels", ["Truck"], "Construct");
			this.templates[type] = new Template("builder", "Viper", "Wheels", ["Truck"], "Construct");
		}

		for (var i in this.availablePropulsions) {
			if (mapTop === 'semi-land' && propulsion[this.availablePropulsions[i]].terrain !== 'water') {
				continue;
			}

			var p = this.availablePropulsions[i];
			template.prop = p;

			if ((!this.preferWater || propulsion[p].terrain !== 'land') && template.speed() > this.templates[type].speed()) {
				this.templates[type] = template;
			}
		}

		for (var i in this.availableBodies) {
			var b = this.availableBodies[i];
			template.body = b;

			var armorType = (Stats.Weapon[this.enemy.weapons[0]].ImpactType === "HEAT" ? "thermal" : "armour");
			if (template.HP() > this.templates[type].HP() && template[armorType]() > this.templates[type][armorType]()) {
				this.templates[type] = template;
			}
		}
	}

	this.designRepair = function() {
		if (!this.templates.main) {
			return;
		}

		if (!countStruct(struct['Command Center'].id , me) || enumStruct(me, struct['Command Center'].id)[0].status === BEING_BUILT) {
			sendRequest('engi', 'Command Center');
			return;
		}

		var t = null;
		if (componentAvailable(turrets.Repair['Heavy Repair Turret'].id)) {
			t = 'Heavy Repair Turret';
		} else if (componentAvailable(turrets.Repair['Repair Turret'].id)) {
			t = 'Repair Turret';
		}

		if (t) {
			this.templates.repair = new Template('repair', this.templates.main.body, this.templates.main.prop, t, 'Repair');
		}
	}

	this.requestResearch = function(type) {
		if (!resCore) { return; }

		var cl = null;
		if (type.match(/strc|bomber/)) {
			cl = 'strc';
		} else if (type.match(/tank|gunship/)) {
			cl = 'tank';
		} else {
			cl = 'pers';
		}

		var classed = wpnCats[type].filter(function(wpn){return Stats.Weapon[wpn].ImpactClass === designer.wpnFocus[cl];});

		if (!this.templates[type]) {
			sendRequest('res', classed[0]);
			return;
		}

		// req upgrades
		for (var i in this.wpnFocus) {
			sendRequest('res', this.wpnFocus[i], 'weapon');
		}
		sendRequest('res', 'droid', 'speed');
		var dmgType = Stats.Weapon[this.enemy.weapons[0]].ImpactType === 'HEAT' ? 'thermal' : 'armour';
		sendRequest('res', 'droid', dmgType);
		sendRequest('res', 'borg', dmgType);


		// req new components
		var template = new Template(type, this.templates[type].body, this.templates[type].prop, this.templates[type].weapons, 'Weapon');
		var best = null;

		for (var i = 0; i < classed.length; i++) {
			template.weapons = [classed[i]];
			if (template.damage(expectedOpponent.type, expectedOpponent.body) > this.templates[type].damage(expectedOpponent.type, expectedOpponent.body)) {
				best = classed[i];
			}
		}

		if (best) {
			sendRequest('res', best);
		}

		for (var p in propulsion) {
			template.prop = p;
			if (this.preferWater && propulsion[p].terrain !== 'land') {
				sendRequest('res', p);
			}
			if (!componentAvailable(propulsion[p].id) && this.enemy.damage(template.prop, template.body) < this.enemy.damage(this.templates[type].prop, this.templates[type].body)) {
				sendRequest('res', p);
				break;
			}
		}

		for (var b in body) {
			if (!componentAvailable(body[b].id) && this.enemy.damage(template.prop, template.body) < this.enemy.damage(this.templates[type].prop, this.templates[type].body)) {
				sendRequest('res', b);
				break;
			}
		}
	}

	this.reviseDesigns = function() {
		this.designBuilder();
		this.designRepair();
		this.designDroid('droid_pers');
		this.designDroid('droid_tank');
		this.designDroid('droid_strc');
		// this.designDroid('raider');
		this.designDroid('anti_air');
		this.designVtol('gunship');
		this.designVtol('bomber');
		// this.designVtol('fighter');
		this.designBorg('borg_pers');
		this.designBorg('borg_tank');
		this.designBorg('borg_strc');
		this.designBorg('borgRepair');
		this.designBorg('borgBuilder');
	}
}