/*
 *	Definition of engineering core, responsible for building construction.
 */

var availableDefense = [];

function EngineeringCore() {
	MacroCore.call(this);

	this.strucInPlan = {
		'Factory': 'prod',
		'Cyborg Factory': 'prod',
		'VTOL Factory': 'prod',
		'Research Facility': 'tech',
		'Command Center': 'prio',
		'Laser Satellite Command Post': 'prio',
		'Oil Derrick': 'prio',
		'Power Generator': 'prio',
		'Satellite Uplink Center': 'prio',
		'VTOL Rearming Pad': 'prio',
		'Repair Facility': 'prio',
		'Power Module': 'prio',
		'Factory Module': 'prio',
		'Research Module': 'tech',
	}
	// single object?
	this.buildValues = {
		'Factory': 0,
		'Cyborg Factory': 0,
		'VTOL Factory': 0,
		'Research Facility': 0,
		'Command Center': 0,
		'Laser Satellite Command Post': 0,
		'Oil Derrick': 0,
		'Power Generator': 0,
		'Satellite Uplink Center': 0,
		'VTOL Rearming Pad': 0,
		'Repair Facility': 0,
		'Power Module': 0,
		'Factory Module': 0,
		'Research Module': 0,
		defense: 0,
	}

	this.buildCost = {
		'Factory': 0,
		'Cyborg Factory': 0,
		'VTOL Factory': 0,
		'Research Facility': 0,
		'Command Center': 0,
		'Laser Satellite Command Post': 1000,
		'Oil Derrick': 0,
		'Power Generator': 0,
		'Satellite Uplink Center': 1000,
		'VTOL Rearming Pad': 0,
		'Repair Facility': 0,
		'Power Module': 0,
		'Factory Module': 0,
		'Research Module': 0,
		'defense': 0
	}

	this.checkpoints = {};

	this.mainBuilders = new BuilderGroup(99);
	this.roamers = new BuilderGroup(98);

	// Someone requested a structure...
	this.request = function(struc, pos) {
		if (pos) { // i know it's defense
			struc = this.pickDefense(availableDefense);

			if (!struc) {
				this.reqNewDefense();
				struc = 'Heavy Machinegun Guard Tower';
			}

			if (!this.checkpoints[pos]) {
				this.checkpoints[pos] = mainCore.increments['turtle'];
			} else {
				this.checkpoints[pos] + mainCore.increments['turtle'];
			}

			if (!this.buildValues[struc]) {
				this.buildValues[struc] = mainCore.increments['turtle'];
			} else {
				this.buildValues[struc] += mainCore.increments['turtle'];
			}
			return;
		}
		if (!isStructureAvailable(struct[struc].id, me)) {
			sendRequest('res', struc);
		}
		if (countStruct(struct[struc].id, me) < getStructureLimit(struct[struc].id)) {
			if (this.strucInPlan[struc] === 'prio') {
				this.buildValues[struc] += maxInObject(mainCore.increments);
			} else {
				this.buildValues[struc] += mainCore.increments[this.strucInPlan[struc]];
			}
		}
	};

	// Fetches idle builder from given group
	this.getIdleBuilder = function(group) {
		var u = group.getUnits();
		for (var i = 0; i < u.length; i++) {
			if (u[i].order !== DORDER_BUILD &&
				u[i].order !== DORDER_LINEBUILD &&
				u[i].order !== DORDER_RECYCLE &&
				u[i].order !== DORDER_DEMOLISH &&
				u[i].order !== DORDER_HELPBUILD &&
				u[i].order !== DORDER_RECOVER) {
				return u[i];
			}
		}

		return null;
	};

	// New builder has been produced, get him to work
	this.incorporateNewBuilder = function(builder) {
		if (this.mainBuilders.size() < 2) { // TODO: make this smarter
			this.mainBuilders.add(builder);
		} else {
			this.roamers.add(builder);
		}
	}

	// Attempts to build give structure, optionally on or around given coordinates. Returns false if unable.
	this.tryBuild = function(structure, idler, x, y) {

		if (structure !== struct['Oil Derrick'].id && this.mainBuilders.status === state.IDLE) {
				return this.mainBuilders.build(structure, x, y);
		} else {
			if (defined(x) && defined(y) && idler) {
				return orderDroidBuild(idler, DORDER_BUILD, structure, x, y);
			}
		}

		return false;
	};

	// Gives priority to basic buildings if they're misssing
	this.ensureBasics = function() {
		if (countStruct(struct['Factory'].id, me) === 0) {
			this.buildValues['Factory'] += maxInObject(mainCore.increments) + 3;
		}
		if (countStruct(struct['Research Facility'].id, me) === 0) {
			this.buildValues['Research Facility'] += maxInObject(mainCore.increments) + 2;
		}
		if (countStruct(struct['Command Center'].id, me) === 0) {
			this.buildValues['Command Center'] += maxInObject(mainCore.increments) + 1;
		}

		var uc = enumStruct(me).filter(function(s){return s.status === BEING_BUILT;});

		if (uc.length) {
			var s = struct[uc[0].name] || defense[uc[0].name];

			if (this.tryBuild(s.id, undefined, uc[0].x, uc[0].y)) {
				return;
			}
		}
	};

	// Finds and builds a new derrick.
	this.expandOilfields = function() {
		var rigs = graph.nearestNode(bases[me], freeDerrick, designer.templates['builder'].prop, this.roamers.size());

		if (!rigs.length && this.roamers.size()) {
			var builder = this.getIdleBuilder(this.roamers);

			if (builder) {
				this.mainBuilders.add(builder);
			}
		}

		for (var r in rigs) {
			var rig = rigs[r];
			var builder = this.getIdleBuilder(this.roamers);

			if (builder) {
				if (this.tryBuild(struct['Oil Derrick'].id, builder, rig.coords.c, rig.coords.r)) {
					rig.type = nodeType.DERRICK_CLAIMED;
					rig.timeClaimed = gameTime;
					queue("checkDerrickBuilt", 60000, rig);
					continue;
				}
			} else {
				if (this.mainBuilders.size() > 2) {
					this.roamers.add(this.mainBuilders.getUnits()[0]);
				} else {
					sendRequest('prod', 'builder', prodCore.unitAmnt.droid.builder ? undefined : 1);
				}
			}
		}

	};

	// Callback for finished construction.
	this.buildingDone = function(structure, group) {
		var name = structure.name;
		this.buildValues[name] = 0;

		if (!this.buildCost[name]) {
			var s = struct[name] || defense[name];
			this.buildCost[name] = enumStruct(me, s.id)[0].cost;
			pmngr.power -= this.buildCost[name];
		}

		if (group === this.mainBuilders.units) {
			this.mainBuilders.status = state.IDLE;
		}
	}

	this.underConstruction = function(struc) {
		var uc = null;
		var arr = enumStruct(me, struc);
		for (var i in arr) {
			if (arr[i].status === BEING_BUILT) {
				uc = struc;
			}
		}
		return uc;
	}

	// Builds the most pressing structure.
	this.build = function() {
		var strucs = this.sortByValue(this.buildValues);
		var struc = null;
		var coords = new Point(null, null);

		for (var i = 0, l = strucs.length; i < l; i++) {
			var s = struct[strucs[i]] || defense[strucs[i]];
			if (!isStructureAvailable(s.id, me)) {
				continue;
			}

			if (!this.canAfford(this.buildCost[strucs[i]])) {
				sendRequest('pmngr', 'engi');
			} else {
				struc = strucs[i];
				break;
			}
		}

		if (!struc) { return; }

		switch (struc) {
			case 'Power Module': {
				var s = enumStruct(me, struct['Power Generator'].id).filter(function(o){return !o.modules;});

				if (s.length) {
					coords.r = s[0].y;
					coords.c = s[0].x;
				} else {
					this.buildValues[struc] = 0;
					return;
				}
				break;
			}
			case 'Factory Module': {
				var s = enumStruct(me, struct['Factory'].id).concat(enumStruct(me, struct['VTOL Factory'].id)).filter(function(o){return o.modules < 2;});

				if (s.length) {
					coords.r = s[0].y;
					coords.c = s[0].x;
				} else {
					this.buildValues[struc] = 0;
					return;
				}
				break;
			}
			case 'Research Module': {
				var s = enumStruct(me, struct['Research Facility'].id).filter(function(o){return !o.modules;});

				if (s.length) {
					coords.r = s[0].y;
					coords.c = s[0].x;
				} else {
					this.buildValues[struc] = 0;
					return;
				}
				break;
			}
			default: {
				if (Stats.Building[struc] && Stats.Building[struc].Type === 'Wall') {
					var p = this.sortByValue(this.checkpoints);

					for (var i = 0; i < p.length; i++) {
						var point = p[0].match(/(.*):(.*)/);

						if (safeDest(me, point[2], point[1]) && graph.findNode(new Point(point[1], point[2]).fortified < FORTIFICATION_LIMIT)) {
							coords.r = point[1];
							coords.c = point[2];
							break;
						}
					}
				}
			}
		}

		var s = struct[struc] || defense[struc];
		if (this.tryBuild(s.id, undefined, coords.c, coords.r)) {
			this.mainBuilders.status = state.ENGAGED;
			pmngr.power -= this.buildCost[struc] ? this.buildCost[struc] : 0;

			if (defense[struc]) {
				this.checkpoints[coords.toString()] = 0;
			}
		}
	}

	// Chooses the best defense.
	this.pickDefense = function(set) {
		var best = set.length ? { name: set[0], worth: weaponWorth(defense[set[0]].gun, expectedOpponent.type, expectedOpponent.body)} : {};

		for (var i = 0; i < set.length; i++) {
			var worth = weaponWorth(defense[set[i]].gun, expectedOpponent.type, expectedOpponent.body);

			if (worth > best.worth) {
				best.name = set[i];
				best.worth = worth;
			}
		}

		return best.name;
	}

	// Request more builders if appropriate
	this.checkBuilders = function() {
		if (!this.mainBuilders.size()) {
			sendRequest('prod', 'builder', prodCore.unitAmnt.droid.builder ? undefined : 1);
		}
	}

	this.reqNewDefense = function() {
		var r = null;
		if (enemyGroups['borgCount'] > enemyGroups['droidCount']) {
			r = defCats[designer.wpnFocus['pers']].filter(function(defname){return findResearch(defense[defname].tech, me).length === 1;});
		} else {
			r = defCats[designer.wpnFocus['tank']].filter(function(defname){return findResearch(defense[defname].tech, me).length === 1;});
		}

		sendRequest('res', this.pickDefense(r));
	}

	// Main process of the core.
	this.process = function() {
		var u = this.mainBuilders.getUnits();
		var idle = true;

		for (var i = 0; i < u.length; i++) {
			if(u[i].order === DORDER_BUILD ||
				u[i].order === DORDER_LINEBUILD ||
				u[i].order === DORDER_RECYCLE ||
				u[i].order === DORDER_DEMOLISH ||
				u[i].order === DORDER_HELPBUILD ||
				u[i].order === DORDER_RECOVER) {idle = false;}
		}

		if (idle) { this.mainBuilders.status = state.IDLE; }

		this.ensureBasics();
		this.expandOilfields();
		this.checkBuilders();
		this.build();
		this.reqNewDefense();
	};
}

EngineeringCore.prototype = Object.create(MacroCore.prototype);
EngineeringCore.prototype.constructor = EngineeringCore;
