/*
 * Event callbacks as defined in API documentation.
 */

/*
 * Figure out map topology, create graph, initialise cores & helper structures
 */
function eventGameInit() {
	mapIdsToNames();
	categorizeWpns();
	categoriseDef();

	for (var i = 0; i < startPositions.length; i++) {
		bases[i] = new Point(startPositions[i].y, startPositions[i].x);
	}

	mapTop = mapType();
	var prop;

	switch(mapTop) {
		case 'land': {
			prop = propulsion.Wheels.id;
			break;
		}
		case 'semi-land':
		case 'water': {
			prop = propulsion.Hover.id;
			break;
		}
		case 'vtol': {
			prop = propulsion.VTOL.id;
			break;
		}
	}

	profile('createMentalMap', prop);


	engiCore = new EngineeringCore();
	resCore = new ResearchCore();
	prodCore = new ProductionCore();
	pmngr = new PowerManager();
	colonel = new ColonelCore();
	mainCore = new MainCore();
}

/*
 * Set up timers
 */
function eventStartLevel() {
	var b = enumDroid(me, DROID_CONSTRUCT);

	b.forEach(function(b) {
		engiCore.mainBuilders.add(b);
	});

	setTimer("updateGroupPos", 500 + randomInt(500));
	setTimer("checkForPickups", 20000 + randomInt(1000));
	setTimer("rationPower", 500 + randomInt(250));
	setTimer("processMacroCores", 2000 + randomInt(1000));
	setTimer("processMicroCores", 1000 + randomInt(1000));
	setTimer("hazardDecay", HAZARD_DECAY_DELAY + randomInt(1000));
	setTimer("noticeAroundNodes", 1500 + randomInt(1000));
	setTimer("analyseEnemyDesign", 10000 + randomInt(1000));
	setTimer("enemyGroupDecay", 1000 + randomInt(1000));
	// setTimer("evalSituation", 60000 + randomInt(1000));
	setTimer('findOutlyingStrucs', 25000 + randomInt(1000));
	setTimer('processMain', 30000 + randomInt(1000));
	mainCore.evalPlan();
	designer.refocus();

	focusedPlayer = lockOnPlayer();
}

function eventResearched(research, structure, player) {
	if (!resToName) { mapResToNames(); }

	var rid = research.id;

	if (resCore) { resCore.resDone(rid); }

	if (!resToName[rid]) { return; }
	for (var i = 0; i < resToName[rid].length; i++) {
		var component = resToName[rid][i];
		if (propulsion[component]) {
			if (component !== 'VTOL' && (!mapTop || mapTop.match(/land/) || propulsion[component].terrain === mapTop)) {
				designer.availablePropulsions.push(component);
				continue;
			}
		}
		if (body[component]) {
			designer.availableBodies.push(component);
			continue;
		}
		if (defense[component]) {
			availableDefense.push(component);
		}

		for (var type in wpnCats) {
			var index = wpnCats[type].indexOf(component);
			if (turrets.Weapon[component] && index >= 0) {
				wpnCats[type].splice(index, 1);
				designer.availableWeapons[type].push(component);
				continue;
			}
		}
	}

	designer.reviseDesigns();
}

function eventDroidIdle(droid) {
	if (droid.droidType === DROID_CONSTRUCT && !droid.group) {
		engiCore.incorporateNewBuilder(droid);
	}
}

function eventDroidBuilt(droid) {
	prodCore.prodDone(droid.propulsion, droid.name);

	if (droid.droidType === DROID_WEAPON || droid.droidType === DROID_CYBORG) {
		colonel.assignDroid(droid);
	}

	if (droid.droidType === DROID_CONSTRUCT) {
		if (!initialScout) {
		} else {
			engiCore.incorporateNewBuilder(droid);
		}
	}
}

function eventStructureBuilt(structure, droid) {
	engiCore.buildingDone(structure, droid.group);

	if (structure.stattype === RESOURCE_EXTRACTOR) {
		if (structure.player !== me) {
			console('built derrick, but someone else beat me to it');
		}
		graph.nodes.at(new Point(structure.y, structure.x)).type = nodeType.DERRICK_BUILT;
	}
}

function eventDestroyed(object) {
	if (object.type === STRUCTURE && object.stattype === RESOURCE_EXTRACTOR) {
		graph.nodes.at(new Point(object.y, object.x)).type = nodeType.DERRICK_FREE;
	}
}

function eventAttacked(victim, attacker) {
	if (attacker.player === me) { return; }

	var e = enumRange(victim.x, victim.y, 10, ENEMIES).filter(isCombatant);
	if (victim.type === DROID) {
		if (!e.length) { return; }

		if (victim.droidType == DROID_CONSTRUCT) {
			orderDroidLoc(victim, DORDER_MOVE, bases[me].c, bases[me].r);
		} else {
			var a = getDroidsArmy(victim);

			if (a) {
				a.respond(e);
			}
		}
	} else if (victim.type === STRUCTURE) {
		// help request
		var units = 0;
		if (victim.stattype === DEFENSE) {
			units = enumRange(victim.x, victim.y, 6, ALLIES).filter(isDefense);
		}
		colonel.sendAssistance({ units: -1, pos: new Point(victim.y, victim.x) }, units, e); // TODO
	}
}

function eventGroupLoss(object, group_id, new_size) {
	if (colonel.sgts[group_id]) {
		var sgt = colonel.sgts[group_id];
		if (new_size === 0) {
			console('sgt lost');
			sgt.status = state.IDLE;

			if (colonel.targets[sgt.target]) {
				delete colonel.targets[sgt.target][sgt.units.units];
			}

			sgt.intention = null;
			sgt.target = null;

			if (sgt.role.match(/borg|main/)) {
				var suggestedRole = colonel.assignRole();

				sgt.role = suggestedRole.join('_');
				sgt.type = suggestedRole[0];
			}

			console(sgt.type);
			sgt.prop = designer.templates[sgt.role] ? designer.templates[sgt.role].prop : designer.templates['droid_pers'].prop;
			prodCore.unitAmnt[sgt.type][sgt.role] += MAX_SQUAD_SIZE;
		} else {
			prodCore.unitAmnt[sgt.type][sgt.role]++;
		}
	}
}

function eventAllianceOffer(from, to) {
	if (to !== me) { return; }
}
