/*
 * Script responsible for making usable graph.
 */

function checkForNode(map, nodes, L, queue, noAdd) {
	var result = [];

	for (var i = 0, l = L.length; i < l; i++) {
		if ((map[L[i].r][L[i].c]) && nodes[L[i].r][L[i].c] !== '-') {
			if (nodes[L[i].r][L[i].c] && nodes[L[i].r][L[i].c].match(/p|s|d|r|g/)) {
				if (!noAdd) {
					graph.addEdge(queue[0], L[i]);
				}
				queue.push(graph.findNode(L[i]));
			} else {
				result.push(L[i]);
			}
		}
	}

	for (var i = 0; i < result.length; i++) {
		nodes[result[i].r][result[i].c] = '-';
	}

	return result;
}

function removeSelfReferences() {
	for (var i = 0, l = graph.allNodes.length; i < l; i++) {
		var node = graph.allNodes[i];

		for (var e in node.edges) {
			if (e === node.coords.toString() || node.edges[e].dist === 0) {
				delete node.edges[e];
			}
		}
	}
}

function pushNeighbours(queue, done, cluster, nodeCoords) {
	var node = graph.nodes.at(nodeCoords);

	for (var i in node.edges) {
		if (manhattanDistance(node.coords, node.edges[i].to) === 1 && !arrayContains(cluster, node.edges[i].to)) {

			cluster.push(node.edges[i].to);
			pushNeighbours(queue, done, cluster, node.edges[i].to);
		} else if (!arrayContains(queue, node.edges[i].to) && !defined(done[i]) && !arrayContains(cluster, node.edges[i].to)) {
			queue.push(node.edges[i].to);
		}
	}
}

function mergeEdges(node, edges) {
	for (var e in edges) {
		if (defined(node.edges[e])) { continue; }

		node.edges[e] = new Edge(edges[e].to, distance(node.coords, edges[e].to));
	}
}

function nodeless(cluster) {
	while (cluster.length > 1) {
		var newNode = graph.nodes.at(cluster[1]);
		var oldNode = graph.nodes.at(cluster[0]);

		mergeEdges(newNode, oldNode.edges);

		for (var i in oldNode.edges) {

			delete graph.nodes.at(oldNode.edges[i].to).edges[oldNode.coords.toString()];

			if (i === oldNode.coords.toString()) { continue; }

			graph.nodes.at(oldNode.edges[i].to).edges[newNode.coords.toString()] = new Edge(newNode.coords, distance(oldNode.edges[i].to, newNode.coords));
		}

		if (cluster.length > 2) {
			var newEndNode = graph.nodes.at(cluster[cluster.length - 2]);
			var oldEndNode = graph.nodes.at(cluster[cluster.length - 1]);

			mergeEdges(newEndNode, oldEndNode.edges);

			for (var i in oldEndNode.edges) {
				delete graph.nodes.at(oldEndNode.edges[i].to).edges[oldEndNode.coords.toString()];

				if (i === oldEndNode.coords.toString()) { continue; }

				graph.nodes.at(oldEndNode.edges[i].to).edges[newEndNode.coords.toString()] = new Edge(newEndNode.coords, distance(oldEndNode.edges[i].to, newEndNode.coords));
			}

			delete newEndNode.edges[oldEndNode.coords.toString()];

			graph.nodes.set(oldEndNode.coords, null);
			cluster.pop();
		}

		delete newNode.edges[oldNode.coords.toString()];

		graph.nodes.set(oldNode.coords, null);
		cluster.shift();
	}
}

function mergeNeighboursButBetter() {
	var queue = [bases[me]];
	var cluster = [];
	var done = {};

	while (queue.length) {
		if (!graph.nodes.at(queue[0])) { queue.shift(); continue; }

		cluster.push(queue[0]);
		pushNeighbours(queue, done, cluster, queue[0]);

		var majorNodes = [];
		for (var i = 0; i < cluster.length; i++) {
			var node = graph.nodes.at(cluster[i]);
			if (node.type !== nodeType.NONE) {
				majorNodes.push(node);
				cluster.splice(i, 1);
				done[node.coords.toString()] = 1;
				i--;
			}
		}

		if (majorNodes.length) {
			while (cluster.length) {
				for (var i = 0; i < cluster.length; i++) {
					var clNodeID = cluster[i].toString();
					var clNode = graph.nodes.at(cluster[i]);
					var merging = false;

					for (var j = 0; j < majorNodes.length; j++) {
						if (majorNodes[j].edges[clNodeID]) {
							merging = true;

							mergeEdges(majorNodes[j], clNode.edges);

							for (var e in clNode.edges) {
								var eNode = graph.nodes.at(clNode.edges[e].to);

								eNode.edges[majorNodes[j].coords.toString()] = new Edge(majorNodes[j].coords, distance(majorNodes[j].coords, eNode.coords));
							}
						}
					}

					if (merging) {
						for (var e in clNode.edges) {
							var eNode = graph.nodes.at(clNode.edges[e].to);

							delete eNode.edges[clNode.coords.toString()];
						}

						graph.nodes.set(clNode.coords, null);
						cluster.splice(i, 1);
						i--;
					}
				}
			}
		} else {
			nodeless(cluster);
			done[cluster.pop().toString()] = 1;
		}

		queue.shift();
	}
}

function neigh8(map, point) {
	var L = [];

	if (isValidTile(map, point.r + 1, point.c)) {
		L.push(new Point(point.r + 1, point.c));
	}
	if (isValidTile(map, point.r - 1, point.c)) {
		L.push(new Point(point.r - 1, point.c));
	}
	if (isValidTile(map, point.r + 1, point.c + 1)) {
		L.push(new Point(point.r + 1, point.c + 1));
	}
	if (isValidTile(map, point.r + 1, point.c - 1)) {
		L.push(new Point(point.r + 1, point.c - 1));
	}
	if (isValidTile(map, point.r, point.c + 1)) {
		L.push(new Point(point.r, point.c + 1));
	}
	if (isValidTile(map, point.r, point.c - 1)) {
		L.push(new Point(point.r, point.c - 1));
	}
	if (isValidTile(map, point.r - 1, point.c + 1)) {
		L.push(new Point(point.r - 1, point.c + 1));
	}
	if (isValidTile(map, point.r - 1, point.c - 1)) {
		L.push(new Point(point.r - 1, point.c - 1));
	}
	return L;
}

function postprocess() {
	/*
	 *	remove unreasonable edges, such as A--C (they lie on the same line)
	 *
	 *	A -- B -- C
	 *	 \_______/
	 */

	for (var i = 0, l = graph.allNodes.length; i < l; i++) {
		var node = graph.allNodes[i];
		for (var e in node.edges) {
			var eNode = graph.nodes.at(node.edges[e].to);
			for (var e2 in eNode.edges) {
				if (node.edges[e2] && angle(node.coords, eNode.coords, eNode.edges[e2].to) < (10 * Math.PI / 180)) {
					// node -> e ; e -> e2 ; node -> e2

					if (node.edges[e2].dist >= eNode.edges[e2].dist && node.edges[e2].dist >= node.edges[e].dist) {
						delete graph.nodes.at(node.edges[e2].to).edges[node.coords.toString()];
						delete node.edges[e2];
					} else if (eNode.edges[e2].dist >= node.edges[e2].dist && eNode.edges[e2].dist >= node.edges[e].dist) {
						delete graph.nodes.at(eNode.edges[e2].to).edges[eNode.coords.toString()];
						delete eNode.edges[e2];
					} else if (node.edges[e].dist >= node.edges[e2].dist && node.edges[e].dist >= eNode.edges[e2].dist) {
						delete eNode.edges[node.coords.toString()];
						delete node.edges[e];
					} else {
						throw 'euclideus would be proud';
					}
				}
			}

			// detect water crossings
			if (mapTop !== 'land' && node.edges[e] && !defined(node.edges[e].water)) {
				var start = null;
				var end = null;

				if (node.coords.r >= eNode.coords.r && node.coords.c >= eNode.coords.c) {
					start = eNode;
					end = node;
				} else if (node.coords.r > eNode.coords.r && node.coords.c < eNode.coords.c || node.coords.r < eNode.coords.r && node.coords.c > eNode.coords.c) {
					start = { coords: new Point(node.coords.r > eNode.coords.r ? eNode.coords.r : node.coords.r, node.coords.c > eNode.coords.c ? eNode.coords.c : node.coords.c) };
					end = { coords: new Point(node.coords.r > eNode.coords.r ? node.coords.r : eNode.coords.r, node.coords.c > eNode.coords.c ? node.coords.c : eNode.coords.c) };
				} else {
					start = node;
					end = eNode;
				}

				var microMap = [];

				for (var j = start.coords.r - 1; j <= start.coords.r + Math.abs(end.coords.r - start.coords.r) + 1; j++) {
					microMap[j] = [];
					for (var k = start.coords.c - 1; k <= start.coords.c + Math.abs(end.coords.c - start.coords.c) + 1; k++) {
						microMap[j][k] = null;
					}
				}

				var microNodes = new Array(mapHeight);
				for (var j = 0; j < mapHeight; j++) {
					microNodes[j] = new Array(mapWidth);
				}

				microNodes[node.coords.r][node.coords.c] = 'p';
				microNodes[eNode.coords.r][eNode.coords.c] = 'p';

				var realEnd = new Point(end.coords.r + 1, end.coords.c + 1);
				processMap(microMap, 'pathmaker', start.coords, realEnd);
				for (var j in microMap[start.coords.r - 1]) {
					microMap[start.coords.r - 1][j] = 0;
				}
				for (var j in microMap[end.coords.r + 1]) {
					microMap[end.coords.r + 1][j] = 0;
				}
				for (var j in microMap) {
					microMap[j][start.coords.c - 1] = 0;
				}
				for (var j in microMap) {
					microMap[j][end.coords.c + 1] = 0;
				}

				thinning(microMap, microNodes, start.coords, realEnd);

				var queue = [node];
				findEdges(microMap, microNodes, queue, 'noAdd');

				if (queue.length !== 2) {
					node.edges[e].water = true;
					eNode.edges[node.coords].water = true;
				} else {
					node.edges[e].water = false;
					eNode.edges[node.coords].water = false;
				}
			}
		}
	}

	for (var i = 0; i < startPositions.length; i++) {
		var pos = new Point(startPositions[i].y, startPositions[i].x);
		var base = graph.nodes.at(pos);

		if (me == i) { bases[me] = new Point(base.coords.r, base.coords.c); }

		bases[i] = new Point(base.coords.r, base.coords.c);

		var derricks = graph.nearestNode(bases[i], unownedFreeDerrick, 'VTOL', 4);

		for (var j = 0; j < derricks.length; j++) {
			if (distance(derricks[j].coords, bases[i]) < 15) {
				derricks[j].owner = i;
			}
		}
	}
}

function findEdges(map, nodes, queue, noAdd) {
	var pathQueue = [];
	var L = neigh8(map, queue[0].coords);

	nodes[queue[0].coords.r][queue[0].coords.c] = '-';
	pathQueue = pathQueue.concat(checkForNode(map, nodes, L, queue, noAdd));

	while(pathQueue.length != 0) {
		L = neigh8(map, pathQueue[0]);
		pathQueue = pathQueue.concat(checkForNode(map, nodes, L, queue, noAdd));
		pathQueue.shift();
	}
}

function makeGraph(map, nodes) {
	var queue = [graph.findNode(bases[me])];

	while (queue.length != 0) {
		findEdges(map, nodes, queue);

		queue.shift();
	}

	mergeNeighboursButBetter();

	// fill node array for easier iteration over all
	for (var r = 0; r < graph.nodes.arr.length; r++) {
		for (var c = 0; c < graph.nodes.arr[r].length; c++) {
			if (graph.nodes.arr[r][c]) {
				if (!Object.keys(graph.nodes.arr[r][c].edges).length) {
					graph.nodes.set(new Point(r, c), null);
					continue;
				}
				var node = graph.nodes.arr[r][c];
				graph.allNodes.push(node);

				// remove self references
				for (var e in node.edges) {
					if (e === node.coords.toString() || node.edges[e].dist === 0) {
						delete node.edges[e];
					}
				}
			}
		}
	}

	postprocess();

	for (var i = 0; i < bases.length; i++) {
		dead[i] = false;
		var ent = findBaseEntrance(i);
		entrance[i] = ent;
		graph.findNode(ent).isChokepoint = true;
	}
}