/*
 *  Definition of production core, responsible for proucing units.
 */

function ProductionCore() {
	MacroCore.call(this);

	this.unitReq = {
		droid: {
			builder: 0,
			droid_pers: 0,
			droid_tank: 0,
			droid_strc: 0,
			raider: 0,
			// arty: 0,
			anti_air: 0,
			repair: 0,
		},
		vtol: {
			bomber: 0,
			gunship: 0,
			fighter: 0,
		},
		borg: {
			borg_pers: 0,
			borg_tank: 0,
			borg_strc: 0,
			borgBuilder: 0,
			borgRepair: 0,
		}
	}
	this.unitAmnt = {
		droid: {
			builder: 0,
			droid_pers: 0,
			droid_tank: 0,
			droid_strc: 0,
			raider: 0,
			// arty: 0,
			anti_air: 0,
			repair: 0,
		},
		vtol: {
			bomber: 0,
			gunship: 0,
			fighter: 0,
		},
		borg: {
			borg_pers: 0,
			borg_tank: 0,
			borg_strc: 0,
			borgBuilder: 0,
			borgRepair: 0,
		}
	}
	this.increments = {
		pers: 5,
		tank: 5,
		strc: 5,
		support: 5
	}

	// Someone requested a unit...
	this.request = function(type, unit, count) {
		var dmg = type.split('_')[1];

		if (!dmg) {
			dmg = 'support';
		}

		this.unitReq[type][unit] += this.increments[dmg];
		if (count) {
			this.unitAmnt[type][unit] += count;
		}
	}

	// Callback for finished production.
	this.prodDone = function(type, template) {
		switch(type) {
			case 'CyborgLegs': {
				this.unitAmnt.borg[template]--;
				if (this.unitReq.borg[template]) {
					this.unitReq.borg[template] = Math.floor(this.unitReq.borg[template] / 2);
				} else {
					this.unitReq.borg[template] = 0;
				}
				break;
			}
			case propulsion.VTOL.id: {
				this.unitAmnt.vtol[template]--;
				if (this.unitReq.vtol[template]) {
					this.unitReq.vtol[template] = Math.floor(this.unitReq.vtol[template] / 2);
				} else {
					this.unitReq.vtol[template] = 0;
				}
				break;
			}
			default: {
				this.unitAmnt.droid[template]--;
				if (this.unitReq.droid[template]) {
					this.unitReq.droid[template] = Math.floor(this.unitReq.droid[template] / 2);
				} else {
					this.unitReq.droid[template] = 0;
				}
			}
		}
	}

	// Wrapper for native produce function.
	this.produceTemplate = function(fact, template) {
		if (template) {
			var wpns = [];
			var b = body[template.body] ? body[template.body].id : cyborgBody[template.body].id;
			var p = propulsion[template.prop] ? propulsion[template.prop].id : cyborgPropulsion[template.prop].id;

			for (var i in template.weapons) {
				wpns.push(turrets[template.wpnType][template.weapons[i]].id);
			}

			produceDroid(fact, template.name, b, p, wpns);
		}
	}

	// Produce a unit. Type denotes unit class: 'droid' | 'vtol' | 'borg'
	this.produce = function(type) {
		var facility = '';
		var fallbackUnit = '';

		switch(type) {
			case 'droid': {
				facility = 'Factory';
				fallbackUnit = designer.templates['droid_tank'] ? 'droid_tank' : 'droid_pers';
				break;
			}
			case 'borg': {
				facility = 'Cyborg Factory';
				fallbackUnit = 'borg_pers';
				break;
			}
			case 'vtol': {
				facility = 'VTOL Factory';
				fallbackUnit = 'gunship';
				break;
			}
		}

		var f = enumStruct(me, struct[facility].id).filter(function(fac) { return fac.status === BUILT });
		var p = this.sortByValue(this.unitReq[type]);
		var busy = 0;
		var qd = {};

		if (!f.length && p.length) {
			for (var i = 0; i < p.length; i++) {
				if (this.unitAmnt[type][p[i]]) {
					sendRequest('engi', facility);
				}
			}
		}

		for (var i = 0; i < f.length; i++) {
			var fac = f[i];
			var unit = getDroidProduction(fac);
			if (unit) {
				if (qd[unit.name]) {
					qd[unit.name]++;
				} else {
					qd[unit.name] = 1;
				}
			}
		};

		for (var j = 0; j < f.length; j++) {
			var fac = f[j];
			if (structureIdle(fac)) {
				var fallbackProd = true;
				for (var i = 0; i < p.length; i++) {
					if (!designer.templates[p[i]]) {
						if (p[i] === 'builder' && enumDroid(me, DROID_CONSTRUCT).length + qd['builder'] > 6) {
							continue;
						}
						if (p[i] === fallbackUnit) {
							fallbackProd = false;
						}

						switch(type) {
							case 'droid': {
								sendRequest('res', wpnCats[p[i]][0]);
								break;
							}
							case 'borg': {
								sendRequest('res', 'Cyborg Factory');
								break;
							}
							case 'vtol': {
								sendRequest('res', 'VTOL');
								sendRequest('res', 'VTOL Factory');
							}
						}
						continue;
					}

					if (this.unitAmnt[type][p[i]] <= 0 || (qd[p[i]] && (this.unitAmnt[type][p[i]] - qd[p[i]]) <= 0)) { continue; }

					if (fac.modules < Stats.Body[designer.templates[p[i]].body].Size) {
						if (p[i] === fallbackUnit) {
							fallbackProd = false;
						}

						sendRequest('engi', 'Factory Module');
						continue;
					}

					if (enumDroid(me, p[i] === 'builder' ? DROID_CONSTRUCT : undefined) === getDroidLimit(me, p[i] === 'builder' ? DROID_CONSTRUCT : undefined) - 1) {
						continue;
					}

					if (this.canAfford(designer.templates[p[i]].cost())) {
						this.produceTemplate(fac, designer.templates[p[i]]);

						if (qd[p[i]]) {
							qd[p[i]]++;
						} else {
							qd[p[i]] = 1;
						}

						pmngr.power -= designer.templates[p[i]].cost();
						fallbackProd = false;
						break;
					} else {
						if (p[i] === fallbackUnit) {
							fallbackProd = false;
						}
						sendRequest('pmngr', 'prod');
						continue;
					}
				}

				if (fallbackProd && designer.templates[fallbackUnit]) {
					this.produceTemplate(fac, designer.templates[fallbackUnit]);
					pmngr.power -= designer.templates[fallbackUnit].cost();
				}
			} else {
				busy++;
			}

			if (fac.modules === 2) {
				sendRequest('res', 'prod', 'speed');
			} else {
				sendRequest('engi', 'Factory Module');
			}
		};

		if (busy && busy === f.length) {
			var cheapest = 99999999;
			for (var i = 0; i < p.length; i++) {
				var cost = designer.templates[p[i]] ? designer.templates[p[i]].cost() : 99999999;
				if (cost < cheapest) {
					cheapest = cost;
				}
			}

			if (pmngr.power >= cheapest) {
				sendRequest('engi', facility);
			}
		}
	}

	// main process of the core
	this.process = function() {
		this.produce('droid');
		this.produce('borg');
		this.produce('vtol');
	}
}

ProductionCore.prototype = Object.create(MacroCore.prototype);
ProductionCore.prototype.constructor = ProductionCore;