/*
 *  Definition of research core, responsible for researching technologies.
 */

function ResearchCore() {
	MacroCore.call(this);

	this.upgrades = {
		weapon: {
			'MACHINE GUN': {
				dmg: "R-Wpn-MG-Damage08",
				rof: "R-Wpn-MG-ROF03"
			},
			'CANNON': {
				dmg: "R-Wpn-Cannon-Damage09",
				rof: "R-Wpn-Cannon-ROF06",
				acc: "R-Wpn-Cannon-Accuracy02"
			},
			'MORTARS': {
				dmg: "R-Wpn-Mortar-Damage06",
				rof: "R-Wpn-Mortar-ROF04",
				acc: "R-Wpn-Mortar-Acc03"
			},
			'MISSILE': {
				dmg: "R-Wpn-Missile-Damage03",
				rof: "R-Wpn-Missile-ROF03",
				acc: "R-Wpn-Missile-Accuracy02"
			},
			'ROCKET': {
				dmg: "R-Wpn-Rocket-Damage09",
				rof: "R-Wpn-Rocket-ROF03",
				acc: "R-Wpn-RocketSlow-Accuracy02"
			},
			'ENERGY': {
				dmg: "R-Wpn-Energy-Damage03",
				rof: "R-Wpn-Energy-ROF03",
				acc: "R-Wpn-Energy-Accuracy01"
			},
			'GAUSS': {
				dmg: "R-Wpn-Rail-Damage03",
				rof: "R-Wpn-Rail-ROF03",
				acc: "R-Wpn-Rail-Accuracy01"
			},
			'FLAME': {
				dmg: "R-Wpn-Flamer-Damage09",
				rof: "R-Wpn-Flamer-ROF03"
			},
			'HOWITZERS': {
				dmg: "R-Wpn-Howitzer-Damage06",
				rof: "R-Wpn-Howitzer-ROF04",
				acc: "R-Wpn-Howitzer-Accuracy03"
			},
			'A-A GUN': {
				dmg: "R-Wpn-AAGun-Damage06",
				rof: "R-Wpn-AAGun-ROF06",
				acc: "R-Wpn-AAGun-Accuracy03"
			},
			'BOMB': {
				dmg: "R-Wpn-Bomb-Accuracy03" // not a mistake, indeed upgrades dmg
			}
		},
		armour: {
			struc: "R-Struc-Materials09",
			droid: "R-Vehicle-Metals09",
			borg: "R-Cyborg-Metals09",
			def: "R-Defense-WallUpgrade12"
		},
		thermal: {
			borg: "R-Cyborg-Armor-Heat09",
			droid: "R-Vehicle-Armor-Heat09"
		},
		speed: {
			res: "R-Struc-Research-Upgrade09",
			rearm: "R-Struc-VTOLPad-Upgrade03",
			prod: "R-Struc-Factory-Upgrade07",
			repair: "R-Struc-RprFac-Upgrade06",
			power: "R-Struc-Power-Upgrade03a",
			droid: "R-Vehicle-Engine09" // horsepower
		},
		autorep: "R-Sys-Autorepair-General"
	};

	this.resValues = { 'R-Wpn-MG1Mk1': 10000 };
	this.pending = {};

	// Someone requested a research...
	this.request = function(obj, type) {
		var t = null;
		var value = 1;
		if (type) {
			t = this.upgrades[type][obj];

			if (type === 'weapon') {
				var cheapest = null;
				var cost = 9999999;
				for (var cat in t) {
					var r = findResearch(t[cat], me);
					if (r.length === 0) {
						continue;
					}

					if (r[0].cost < cost) {
						cheapest = r[0].id;
						cost = r[0].cost;
					}
				}

				t = cheapest;
			}
		} else {
			t = struct[obj] || turrets.Weapon[obj] || propulsion[obj] || body[obj] || defense[obj];

			if (!t) { return; }

			t = t.tech;
		}

		if (!t) { return; }

		// defense advantage
		if (t.match(/^R-Defense-/) || t.match(/Materials/) || t.match(/^R-Vehicle-(Prop|Body)/)) {
			value = mainCore.increments['turtle'];
		}

		// quality advantage
		if (t.match(/Damage|ROF|Accuracy|Metals|Heat/) && (t.match(/^R-Wpn-/) || t.match(/^R-Vehicle-(Prop|Body)/))) {
			value = mainCore.increments['tech'];
		}

		// macro advantage
		if (t.match(/(^R-Struc)|Module|Upgrade/)) {
			value = maxInObject(mainCore.increments);
		}

		if (defined(this.resValues[t])) {
			this.resValues[t] += value;
		} else {
			this.resValues[t] = value;
		}
	}

	// Callback for when research finishes
	this.resDone = function(res) {
		this.resValues[res] = 0;
		delete this.pending[res];
	}

	this.planResearch = function() {

	}

	// Main process of the core.
	this.process = function() {
		var f = enumStruct(me, struct['Research Facility'].id).filter(function(fac) { return fac.status === BUILT });
		var t = this.sortByValue(this.resValues);

		var busy = 0;

		f.forEach(function(fac) {
			if (structureIdle(fac)) {
				for (var i = 0; i < t.length; i++) {
					if (!t[i]) { continue; }

					var r = findResearch(t[i], me);
					if (r.length !== 0) {
						if (defined(resCore.pending[r[r.length - 1]])) { continue; }
						if (resCore.canAfford(r[r.length - 1].power)) {
							resCore.pending[r[r.length - 1].id] = 1;
							pursueResearch(fac, r[r.length - 1].id);
							resCore.resValues[t[i]] = Math.floor(resCore.resValues[t[i]]/2);
							pmngr.power -= r[r.length - 1].power;
							break;
						} else {
							sendRequest('pmngr', 'res');
						}
					}
				}
			} else {
				busy++;
			}

			if (fac.modules) {
				sendRequest('res', 'res', 'speed');
			} else {
				sendRequest('engi', 'Research Module');
			}
		});

		if (busy && busy === f.length) {
			if (mainCore.currentPlan === 'tech') {
				sendRequest('pmngr', 'res');
			}

			var cheapest = 99999999;
			for (var i = 0; i < t.length; i++) {
				if (!t[i]) { continue; }

				var r = profile('findResearch', t[i], me);
				if (r.length !== 0) {
					if (cheapest > r[r.length - 1].power) {
						cheapest = r[r.length - 1].power;
					}
				}
			}

			if (pmngr.power >= cheapest) {
				sendRequest('engi', 'Research Facility');
			}
		}
	}
}

ResearchCore.prototype = Object.create(MacroCore.prototype);
ResearchCore.prototype.constructor = ResearchCore;