/*
 * Generally useful functions.
 */

function Point(i, j) {
	this.r = i;
	this.c = j;
}

Point.prototype = {
	isEqual: function(other) {
		if (other == null) {
			return false;
		}
		return other.r === this.r && other.c === this.c;
	},

	toString: function() {
		return this.r + ":" + this.c;
	}
}

/*
 * Checks tile validity
 */
function isValidTile(map, i, j) {
	return 	   i >= 1
			&& j >= 1
			&& i <= map.length - 1
			&& j <= map[i].length - 1;
}

function log(message) {
	dump(/*'[' + gameTime + "]: " + */message);
}

function defined(a) {
	return typeof a !== 'undefined' && a !== null;
}

function isVisible(object) {
	// first pair of coordinates is inclusive, second pair exclusive
	return enumArea(object.x, object.y, object.x + 1, object.y + 1, ENEMIES).length != 0;
}

function isCombatant(o) {
	return o.droidType === DROID_WEAPON || o.droidType === DROID_CYBORG || o.stattype === DEFENSE;
}

function isDefense(o) {
	return o.stattype === DEFENSE || o.stattype === WALL;
}

function isProduction(o) {
	return o.stattype === FACTORY || o.stattype === CYBORG_FACTORY || o.stattype === VTOL_FACTORY;
}

function isGenerator(o) {
	return o.stattype === POWER_GEN;
}

function isBuilt(s) {
	return s.status === BUILT;
}

function randomInt(max) {
  return Math.floor(Math.random() * Math.floor(max));
}

function powerPerMinute() {
	var per = Upgrades[me].Building["Power Generator"].PowerPoints;
	var moduleBonus = Upgrades[me].Building["Power Generator"].ModulePowerPoints;
	var moduledSlots = 0;
	var derricks = countStruct(struct['Oil Derrick'].id, me);
	var gain = 0;

	var s = enumStruct(me, struct['Power Generator'].id);
	for (var i = 0; i < s.length; i++) {
		if (s[i].modules == 1) {
			moduledSlots += 4;
		}
	}

	for (var i = 0; i < countStruct(struct['Power Generator'].id, me) * 4 || !derricks; i++,  derricks--) {
		if (moduledSlots != 0) {
			gain += per + moduleBonus;
			moduledSlots--;
			continue;
		}
		gain += per;
	}
	return gain;
}

/*
 * Computes minimum in array, which will always have at least 2 elements
 */
function min(array) {
	var min = 1000; // not strictly correct, but suffices

	for (var i = 0; i < array.length; i++) {
		if (array[i] < min) min = array[i];
	}

	return min;
}

/*
 * Computes maximum in array, which will always have at least 2 elements
 */
function max(array) {
	var max = -999999;

	for (var i = 0; i < array.length; i++) {
		if (array[i] > max) max = array[i];
	}

	return max;
}

function maxInObject(obj) {
	var arr = [];
	for (var i in obj) {
		arr.push(obj[i]);
	}
	return max(arr);
}

/*
 * Returns true if given array contains given item (base types / objects with isEqual method ONLY)
 */
function arrayContains(array, item) {
	for (var i = 0; i < array.length; i++) {
		if (defined(item.isEqual) && item.isEqual(array[i])) {
			return true;
		}
		else if (array[i] === item) {
			return true;
		}
	}

	return false;
}

/*
 * Removes all occurences of item from given array (base types / objects with isEqual method ONLY)
 */
function arrayReplace(array, oldItem, newItem) {
	for (var i = array.length - 1; i >= 0; i--) {
		if (defined(oldItem.isEqual) && oldItem.isEqual(array[i])) {
			array[i] = newItem;
		} else if (array[i] === oldItem) {
			array[i] = newItem;
		}
	}
}

/*
 * Copies contents of an array to another one (base types ONLY)
 */
function arrayCopy(from, to) {
	for (var i in from) {
		for (var j in from[i]) {
			// if (!to[i]) { to[i] = []; }
			to[i][j] = from[i][j];
		}
	}
}

function stringContains(source, str) {
	return RegExp('.*' + str + '.*').test(source);
}

function distance(point1, point2) {
	return distBetweenTwoPoints(point1.c, point1.r, point2.c, point2.r);
}

function manhattanDistance(point1, point2) {
	if (point1 == null || point2 == null) {
		return -1;
	}

	return Math.abs(point1.r - point2.r) + Math.abs(point1.c - point2.c);
}

function angle(base, point1, point2) {
	var vector1 = new Point(point1.r - base.r, point1.c - base.c);
	var vector2 = new Point(point2.r - base.r, point2.c - base.c);
	var dotProd = vector1.r * vector2.r + vector1.c * vector2.c;
	var v1l = Math.sqrt(vector1.r * vector1.r + vector1.c * vector1.c); // ECMA 5.1 does not have ** operator :(
	var v2l = Math.sqrt(vector2.r * vector2.r + vector2.c * vector2.c);

	return Math.acos(dotProd/(v1l*v2l));
}

var idToName = null;
var resToName = null;

function mapIdsToNames() {
	idToName = {};

	for (var type in Stats) {
		if (!type.match(/Class/)) {
			for (var item in Stats[type]) {
				idToName[Stats[type][item].Id] = item;
			}
		}
	}

	idToName['B2JeepBody'] = '*Baba Jeep Body*'; // temporary
}

function mapResToNames() {
	resToName = {};
	for (var s in struct) {
		if (resToName[struct[s].tech]) {
			resToName[struct[s].tech].push(s);
		} else {
			resToName[struct[s].tech] = [s];
		}
	}
	for (var p in propulsion) {
		if (resToName[propulsion[p].tech]) {
			resToName[propulsion[p].tech].push(p);
		} else {
			resToName[propulsion[p].tech] = [p];
		}
	}
	for (var b in body) {
		if (resToName[body[b].tech]) {
			resToName[body[b].tech].push(b);
		} else {
			resToName[body[b].tech] = [b];
		}
	}
	for (var d in defense) {
		if (resToName[defense[d].tech]) {
			resToName[defense[d].tech].push(d);
		} else {
			resToName[defense[d].tech] = [d];
		}
	}
	for (var type in turrets) {
		for (var w in turrets[type]) {
			if (resToName[turrets[type][w].tech]) {
				resToName[turrets[type][w].tech].push(w);
			} else {
				resToName[turrets[type][w].tech] = [w];
			}
		}
	}
	for(var b in cyborgBody) {
		if (resToName[cyborgBody[b].tech]) {
			resToName[cyborgBody[b].tech].push(b);
		} else {
			resToName[cyborgBody[b].tech] = [b];
		}
	}
	for(var p in cyborgPropulsion) {
		if (resToName[cyborgPropulsion[p].tech]) {
			resToName[cyborgPropulsion[p].tech].push(p);
		} else {
			resToName[cyborgPropulsion[p].tech] = [p];
		}
	}
}

function weaponWorth(weapon, targetProp, targetBody) {
	var turrobj = turrets.Weapon[weapon] || turrets.Defense[weapon];
	var armorType = (Stats.Weapon[weapon].ImpactType === "HEAT" ? "Thermal" : "Armour");
	var dmgAfterArmor = Upgrades[me].Weapon[weapon].Damage * turrobj.modifiers[targetProp] - Stats.Body[targetBody][armorType];
	var minDmg = Upgrades[me].Weapon[weapon].Damage * turrobj.modifiers[targetProp] * Stats.Weapon[weapon].MinimumDamage / 100;
	var realDmg = ((dmgAfterArmor > minDmg) ? dmgAfterArmor : minDmg);
	var worth = realDmg * Upgrades[me].Weapon[weapon].HitChance/100;

	return worth;
}

function pushByTime(obj, what, where) {
	var arr = obj[where];

	if (!arr.length) {
		arr.push(what);
		return;
	}

	var wpn = turrets.Weapon[what] || defense[what];
	var length = findResearch(wpn.tech, me).length;
	for (var i = 0; i < arr.length; i++) {
		var wpn2 = turrets.Weapon[arr[i]] || defense[arr[i]];
		if (length < findResearch(wpn2.tech, me).length) {
			arr.splice(i, 0, what);
			return;
		}
	}
	arr.push(what);
}