/*
 * Droid turrets
 */


/*
 *	DMG modifiers as defined here:
 *	http://betaguide.wz2100.net/stats.php?view=WeaponPropulsionModifiers
 *	http://betaguide.wz2100.net/stats.php?view=WeaponStructureModifiers
 */
const dmgModifiers = {
	All_Rounder: {
		Wheels: 1.25,
		"Half-tracks": 1.15,
		Tracks: 1.05,
		Hover: 1.2,
		VTOL: 0.4,
		'Cyborg Propulsion': 0.65,
		'strucMedium': 1,
		'strucHard': 1,
		'strucBunker': 0.75
	},
	Anti_Personnel: {
		Wheels: 1,
		"Half-tracks": 0.5,
		Tracks: 0.4,
		Hover: 1.1,
		VTOL: 0.6,
		'Cyborg Propulsion': 1.5,
		'strucMedium': 0.65,
		'strucHard': 0.4,
		'strucBunker': 0.5
	},
	Anti_Tank: {
		Wheels: 1.3,
		"Half-tracks": 1.25,
		Tracks: 1.2,
		Hover: 0.9,
		VTOL: 0.8,
		'Cyborg Propulsion': 0.3,
		'strucMedium': 0.5,
		'strucHard': 0.25,
		'strucBunker': 0.6
	},
	Artillery: {
		Wheels: 0.9,
		"Half-tracks": 0.65,
		Tracks: 0.4,
		Hover: 1.1,
		VTOL: 0.25,
		'Cyborg Propulsion': 1.3,
		'strucMedium': 1.2,
		'strucHard': 1,
		'strucBunker': 0.4
	},
	Bunker_Buster: {
		Wheels: 0.3,
		"Half-tracks": 0.4,
		Tracks: 0.5,
		Hover: 0.2,
		VTOL: 0.3,
		'Cyborg Propulsion': 0.3,
		'strucMedium': 1.2,
		'strucHard': 3,
		'strucBunker': 4
	},
	Flamer: {
		Wheels: 1.1,
		"Half-tracks": 1,
		Tracks: 0.9,
		Hover: 1.3,
		VTOL: 0.25,
		'Cyborg Propulsion': 1.3,
		'strucMedium': 0.6,
		'strucHard': 0.1,
		'strucBunker': 3
	},
	None: {
		Wheels: 0,
		"Half-tracks": 0,
		Tracks: 0,
		Hover: 0,
		VTOL: 0,
		'Cyborg Propulsion': 0,
		'strucMedium': 0,
		'strucHard': 0,
		'strucBunker': 0
	}
}

const unitTypes = {
	Droid: 0,
	Cyborg: 1,
	VTOL: 2
}

/*
 * Turret components for droid design. Object mimicks Stats structure for easier access
 *
 * Structure:
 *	id : string; game ID to pass into API functions
 * 	tech : string; ID of tech needed to unlock this
 * 	modType : string; NAME of applicable modifier
 * 	modifiers : Object; reference into <dmgModifiers> structure
 * 	unitType : integer; type of unit able to wield the turret, see <unitTypes> structure
 * 	*body : string; expected when unitType == Cyborg to link weapons to bodies
 * 	*AA_body : bool; turret can only shoot at air targets
*/
const turrets = {
	Weapon: {
		"Machinegun": 					{ id: "MG1Mk1", tech: "R-Wpn-MG1Mk1", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Droid},
		"VTOL Machinegun": 				{ id: "MG1-VTOL", tech: "R-Wpn-MG1Mk1", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.VTOL},
		"Machinegunner": 				{ id: "CyborgChaingun", tech: "R-Wpn-MG1Mk1", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Flamer": 						{ id: "Flame1Mk1", tech: "R-Wpn-Flamer01Mk1", modType: "Flamer", modifiers: dmgModifiers['Flamer'], unitType: unitTypes.Droid},
		"Cyborg Flamer": 				{ id: "CyborgFlamer01", tech: "R-Wpn-Flamer01Mk1", modType: "Flamer", modifiers: dmgModifiers['Flamer'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Light Cannon": 				{ id: "Cannon1Mk1", tech: "R-Wpn-Cannon1Mk1", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid},
		"VTOL Cannon": 					{ id: "Cannon1-VTOL", tech: "R-Wpn-Cannon1Mk1", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.VTOL},
		"Heavy Gunner": 				{ id: "CyborgCannon", tech: "R-Wpn-Cannon1Mk1", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Twin Machinegun": 				{ id: "MG2Mk1", tech: "R-Wpn-MG2Mk1", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Droid},
		"VTOL Twin Machinegun": 		{ id: "MG2-VTOL", tech: "R-Wpn-MG2Mk1", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.VTOL},
		"Mini-Rocket Pod": 				{ id: "Rocket-Pod", tech: "R-Wpn-Rocket05-MiniPod", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid},
		"VTOL Mini-Rocket": 			{ id: "Rocket-VTOL-Pod", tech: "R-Wpn-Rocket05-MiniPod", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.VTOL},
		"Hurricane AA Turret": 			{ id: "QuadMg1AAGun", tech: "R-Wpn-AAGun03", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid, AA_only: true},
		"Heavy Machinegun": 			{ id: "MG3Mk1", tech: "R-Wpn-MG3Mk1", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Droid},
		"VTOL Heavy Machinegun": 		{ id: "MG3-VTOL", tech: "R-Wpn-MG3Mk1", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.VTOL},
		"Mortar": 						{ id: "Mortar1Mk1", tech: "R-Wpn-Mortar01Lt", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"*Cyborg Grenade Weapon*": 		{ id: "Cyb-Wpn-Grenade", tech: "R-Wpn-Mortar01Lt", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Medium Cannon": 				{ id: "Cannon2A-TMk1", tech: "R-Wpn-Cannon2Mk1", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid},
		"Super Heavy-Gunner": 			{ id: "Cyb-Hvywpn-Mcannon", tech: "R-Cyborg-Hvywpn-Mcannon", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Cyborg, body: "Cyborg Heavy Body"},
		"Mini-Rocket Array": 			{ id: "Rocket-MRL", tech: "R-Wpn-Rocket02-MRL", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"Hyper Velocity Cannon": 		{ id: "Cannon4AUTOMk1", tech: "R-Wpn-Cannon4AMk1", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid},
		"VTOL Hyper Velocity Cannon": 	{ id: "Cannon4AUTO-VTOL", tech: "R-Wpn-Cannon4AMk1", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.VTOL},
		"Super HPV Cyborg": 			{ id: "Cyb-Hvywpn-HPV", tech: "R-Cyborg-Hvywpn-HPV", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Cyborg, body: "Cyborg Heavy Body"},
		"Lancer": 						{ id: "Rocket-LtA-T", tech: "R-Wpn-Rocket01-LtAT", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid},
		"VTOL Lancer": 					{ id: "Rocket-VTOL-LtA-T", tech: "R-Wpn-Rocket01-LtAT", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.VTOL},
		"Cyborg Lancer": 				{ id: "CyborgRocket", tech: "R-Wpn-Rocket01-LtAT", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Bunker Buster": 				{ id: "Rocket-BB", tech: "R-Wpn-Rocket03-HvAT", modType: "Bunker_Buster", modifiers: dmgModifiers['Bunker_Buster'], unitType: unitTypes.Droid},
		"VTOL Bunker Buster": 			{ id: "Rocket-VTOL-BB", tech: "R-Wpn-Rocket03-HvAT", modType: "Bunker_Buster", modifiers: dmgModifiers['Bunker_Buster'], unitType: unitTypes.VTOL},
		"Sunburst AA": 					{ id: "Rocket-Sunburst", tech: "R-Wpn-Sunburst", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid, AA_only: true},
		"VTOL Sunburst AA": 			{ id: "Rocket-VTOL-Sunburst", tech: "R-Wpn-Sunburst", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.VTOL, AA_only: true},
		"Inferno": 						{ id: "Flame2", tech: "R-Wpn-Flame2", modType: "Flamer", modifiers: dmgModifiers['Flamer'], unitType: unitTypes.Droid},
		"*Cyborg Thermite Weapon*": 	{ id: "Cyb-Wpn-Thermite", tech: "R-Wpn-Flame2", modType: "Flamer", modifiers: dmgModifiers['Flamer'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Bombard": 						{ id: "Mortar2Mk1", tech: "R-Wpn-Mortar02Hvy", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"VTOL Cluster Bomb Bay": 		{ id: "Bomb1-VTOL-LtHE", tech: "R-Wpn-Bomb01", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.VTOL},
		"AA Flak Cannon": 				{ id: "AAGun2Mk1", tech: "R-Wpn-AAGun02", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid, AA_only: true},
		"Incendiary Mortar": 			{ id: "Mortar-Incenediary", tech: "R-Wpn-Mortar-Incenediary", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"Assault Cannon": 				{ id: "Cannon5VulcanMk1", tech: "R-Wpn-Cannon5", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid},
		"VTOL Assault Cannon": 			{ id: "Cannon5Vulcan-VTOL", tech: "R-Wpn-Cannon5", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.VTOL},
		"Super Auto-Cannon Cyborg": 	{ id: "Cyb-Hvywpn-Acannon", tech: "R-Cyborg-Hvywpn-Acannon", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Cyborg, body: "Cyborg Heavy Body"},
		"VTOL Phosphor Bomb Bay": 		{ id: "Bomb3-VTOL-LtINC", tech: "R-Wpn-Bomb03", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.VTOL},
		"Ripple Rockets": 				{ id: "Rocket-IDF", tech: "R-Wpn-Rocket06-IDF", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"Assault Gun": 					{ id: "MG4ROTARYMk1", tech: "R-Wpn-MG4", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Droid},
		"VTOL Assault Gun": 			{ id: "MG4ROTARY-VTOL", tech: "R-Wpn-MG4", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.VTOL},
		"*CyborgRotMG*": 				{ id: "CyborgRotMG", tech: "R-Wpn-MG4", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Heavy Cannon": 				{ id: "Cannon375mmMk1", tech: "R-Wpn-Cannon3Mk1", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid},
		"Whirlwind AA Turret": 			{ id: "QuadRotAAGun", tech: "R-Wpn-AAGun04", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid, AA_only: true},
		"VTOL Heap Bomb Bay": 			{ id: "Bomb2-VTOL-HvHE", tech: "R-Wpn-Bomb02", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.VTOL},
		"Twin Assault Cannon":			{ id: "Cannon6TwinAslt", tech: "R-Wpn-Cannon6TwinAslt", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid},
		"VTOL Thermite Bomb Bay": 		{ id: "Bomb4-VTOL-HvyINC", tech: "R-Wpn-Bomb04", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.VTOL},
		"Howitzer": 					{ id: "Howitzer105Mk1", tech: "R-Wpn-HowitzerMk1", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"Plasma Cannon": 				{ id: "Laser4-PlasmaCannon", tech: "R-Wpn-PlasmaCannon", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid},
		"Tank Killer": 					{ id: "Rocket-HvyA-T", tech: "R-Wpn-Rocket07-Tank-Killer", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid},
		"VTOL Tank Killer": 			{ id: "Rocket-VTOL-HvyA-T", tech: "R-Wpn-Rocket07-Tank-Killer", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.VTOL},
		"Super Tank-Killer Cyborg": 	{ id: "Cyb-Hvywpn-TK", tech: "R-Cyborg-Hvywpn-TK", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Cyborg, body: "Cyborg Heavy Body"},
		"Pepperpot": 					{ id: "Mortar3ROTARYMk1", tech: "R-Wpn-Mortar3", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"Plasmite Flamer": 				{ id: "PlasmiteFlamer", tech: "R-Wpn-Plasmite-Flamer", modType: "Flamer", modifiers: dmgModifiers['Flamer'], unitType: unitTypes.Droid},
		"Flashlight": 					{ id: "Laser3BEAMMk1", tech: "R-Wpn-Laser01", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Droid},
		"VTOL Flashlight": 				{ id: "Laser3BEAM-VTOL", tech: "R-Wpn-Laser01", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.VTOL},
		"*Cyb-Wpn-Laser*": 				{ id: "Cyb-Wpn-Laser", tech: "R-Wpn-Laser01", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Needle Gun": 					{ id: "RailGun1Mk1", tech: "R-Wpn-RailGun01", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid},
		"VTOL Needle Gun": 				{ id: "RailGun1-VTOL", tech: "R-Wpn-RailGun01", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.VTOL},
		"*Cyb-Wpn-Rail1*": 				{ id: "Cyb-Wpn-Rail1", tech: "R-Wpn-RailGun01", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Twin Assault Gun": 			{ id: "MG5TWINROTARY", tech: "R-Wpn-MG5", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Droid},
		"Scourge Missile": 				{ id: "Missile-A-T", tech: "R-Wpn-Missile2A-T", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid},
		"VTOL Scourge Missile": 		{ id: "Missile-VTOL-AT", tech: "R-Wpn-Missile2A-T", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.VTOL},
		"*Cyb-Wpn-Atmiss*": 			{ id: "Cyb-Wpn-Atmiss", tech: "R-Wpn-Missile2A-T", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Cyborg, body: "Cyborg Light Body"},
		"Super Scourge Cyborg": 		{ id: "Cyb-Hvywpn-A-T", tech: "R-Cyborg-Hvywpn-A-T", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Cyborg, body: "Cyborg Heavy Body"},
		"Ground Shaker": 				{ id: "Howitzer150Mk1", tech: "R-Wpn-HvyHowitzer", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"VTOL Plasmite Bomb Bay": 		{ id: "Bomb5-VTOL-Plasmite", tech: "R-Wpn-Bomb05", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.VTOL},
		"Avenger SAM": 					{ id: "Missile-LtSAM", tech: "R-Wpn-Missile-LtSAM", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid, AA_only: true},
		"Seraph Missile Array": 		{ id: "Missile-MdArt", tech: "R-Wpn-MdArtMissile", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"Incendiary Howitzer": 			{ id: "Howitzer-Incenediary", tech: "R-Wpn-Howitzer-Incenediary", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"Hellstorm": 					{ id: "Howitzer03-Rot", tech: "R-Wpn-Howitzer03-Rot", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"Pulse Laser": 					{ id: "Laser2PULSEMk1", tech: "R-Wpn-Laser02", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Droid},
		"VTOL Pulse Laser": 			{ id: "Laser2PULSE-VTOL", tech: "R-Wpn-Laser02", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.VTOL},
		"Super Pulse Laser Cyborg": 	{ id: "Cyb-Hvywpn-PulseLsr", tech: "R-Cyborg-Hvywpn-PulseLsr", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Cyborg, body: "Cyborg Heavy Body"},
		"Heavy Laser": 					{ id: "HeavyLaser", tech: "R-Wpn-HvyLaser", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.Droid},
		"VTOL Heavy Laser": 			{ id: "HeavyLaser-VTOL", tech: "R-Wpn-HvyLaser", modType: "Anti_Personnel", modifiers: dmgModifiers['Anti_Personnel'], unitType: unitTypes.VTOL},
		"Stormbringer AA Laser": 		{ id: "AAGunLaser", tech: "R-Wpn-AALaser", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid, AA_only: true},
		"Rail Gun": 					{ id: "RailGun2Mk1", tech: "R-Wpn-RailGun02", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid},
		"VTOL Rail Gun": 				{ id: "RailGun2-VTOL", tech: "R-Wpn-RailGun02", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.VTOL},
		"Super Rail-Gunner": 			{ id: "Cyb-Hvywpn-RailGunner", tech: "R-Cyborg-Hvywpn-RailGunner", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Cyborg, body: "Cyborg Heavy Body"},
		"EMP Cannon": 					{ id: "EMP-Cannon", tech: "R-Wpn-EMPCannon", modType: "None", modifiers: dmgModifiers['None'], unitType: unitTypes.Droid},
		"Archangel Missile": 			{ id: "Missile-HvyArt", tech: "R-Wpn-HvArtMissile", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.Droid},
		"VTOL EMP Missile Launcher": 	{ id: "Bomb6-VTOL-EMP", tech: "R-Wpn-Bomb06", modType: "Artillery", modifiers: dmgModifiers['Artillery'], unitType: unitTypes.VTOL},
		"Vindicator SAM": 				{ id: "Missile-HvySAM", tech: "R-Wpn-Missile-HvSAM", modType: "Anti_Tank", modifiers: dmgModifiers['Anti_Tank'], unitType: unitTypes.Droid, AA_only: true},
		"Gauss Cannon": 				{ id: "RailGun3Mk1", tech: "R-Wpn-RailGun03", modType: "All_Rounder", modifiers: dmgModifiers['All_Rounder'], unitType: unitTypes.Droid}
	},

	Defense: {
		'Heavy Machinegun Bunker': {id: 'MG3-Pillbox', tech: 'R-Defense-Pillbox01', modType: 'Anti_Personnel', modifiers: dmgModifiers['Anti_Personnel'] },
		'*MG4ROTARY-Pillbox*': {id: 'MG4ROTARY-Pillbox', tech: 'R-Defense-RotMG', modType: 'Anti_Personnel', modifiers: dmgModifiers['Anti_Personnel'] },
		'Cannon Fortress': {id: 'CannonSuper', tech: 'R-Defense-Super-Cannon', modType: 'All_Rounder', modifiers: dmgModifiers['All_Rounder'] },
		'Heavy Rocket Bastion': {id: 'RocketSuper', tech: 'R-Defense-Super-Rocket', modType: 'Anti_Tank', modifiers: dmgModifiers['Anti_Tank'] },
		'Mass Driver': {id: 'MassDriver', tech: 'R-Defense-MassDriver', modType: 'All_Rounder', modifiers: dmgModifiers['All_Rounder'] },
		'Missile Fortress': {id: 'MissileSuper', tech: 'R-Defense-Super-Missile', modType: 'Anti_Tank', modifiers: dmgModifiers['Anti_Tank'] },
	},

	Construct: {
		Truck: 					{ id: "Spade1Mk1", tech: "R-Sys-Spade1Mk1", modType: "None", modifiers: dmgModifiers['None'], unitType: unitTypes.Droid},
		"*Combat Engineer*": 	{ id: "CyborgSpade", tech: "R-Sys-Engineering01", unitType: unitTypes.Cyborg, body: "Cyborg Light Body"}
	},

	Repair: {
		"Repair Turret": 		{ id: "LightRepair1", tech: "R-Sys-MobileRepairTurret01", unitType: unitTypes.Droid},
		"Heavy Repair Turret": 	{ id: "HeavyRepair", tech: "R-Sys-MobileRepairTurretHvy", unitType: unitTypes.Droid},
		"*Cyborg Mechanic*": 	{ id: "CyborgRepair", tech: "R-Sys-MobileRepairTurret01", unitType: unitTypes.Cyborg, body: "Cyborg Light Body"}
	},

	// unordered, need renaming
	Sensor: {
		CB: { id: "Sys-CBTurret01", tech: "R-Sys-CBSensor-Turret01", unitType: unitTypes.Droid},
		VTOLCB: { id: "Sys-VTOLCBTurret01", tech: "R-Sys-VTOLCBS-Turret01", unitType: unitTypes.VTOL},
		RadarDetector: { id: "RadarDetector", tech: "R-Sys-RadarDetector01", unitType: unitTypes.Droid},
		Sensor: { id: "SensorTurret1Mk1", tech: "R-Sys-Sensor-Turret01", unitType: unitTypes.Droid},
		VTOLStrike: { id: "Sys-VstrikeTurret01", tech: "R-Sys-VTOLStrike-Turret01", unitType: unitTypes.VTOL},
		WideSpectrum: { id: "Sensor-WideSpec", tech: "R-Sys-Sensor-WS", unitType: unitTypes.Droid}
	}
}

const speshul = {
	// transports are made of body only
	"Transport Body": { id: "TransporterBody", tech: "R-Cyborg-Transport"},
	"Super Transport Body": { id: "SuperTransportBody", tech: "R-SuperTransport"},
}

var wpnCats = {
	droid_pers: [],
	droid_tank: [],
	droid_strc: [],
	gunship: [],
	bomber: [],
	fighter: [], // containment for AA only weapons
	anti_air: [], // containment for AA only weapons
	borg_pers: [],
	borg_tank: [],
	borg_strc: []
}

/*
 *	Sorts weapons intocategories based on what unit class should use them.
 */
function categorizeWpns() {
	for (var wpn in turrets.Weapon) {
		var w = turrets.Weapon[wpn];
		// log(wpn);
		switch (w.unitType) {
			case unitTypes.Droid: {
				if (w.AA_only) {
					// log('--to anti_air');
					pushByTime(wpnCats, wpn, 'anti_air');
					break;
				}
				if (w.modifiers['strucMedium'] >= 1 || w.modifiers['strucHard'] >= 1 || w.modifiers['strucBunker'] >= 1) {
					// log('--to arty');
					pushByTime(wpnCats, wpn, 'droid_strc');
				}
				if (w.modifiers['Cyborg Propulsion'] >= 1) {
					// log('--to raider');
					pushByTime(wpnCats, wpn, 'droid_pers');
				}
				// log('--to main');
				if (w.modifiers['Wheels'] + w.modifiers['Half-tracks'] + w.modifiers['Tracks'] + w.modifiers['Hover'] >= 4) {
					pushByTime(wpnCats, wpn, 'droid_tank');
				}
				break;
			}
			case unitTypes.VTOL: {
				if (w.AA_only) {
					// log('--to fighter');
					pushByTime(wpnCats, wpn, 'fighter');
					break;
				}
				if (w.modifiers['strucMedium'] > 1 || w.modifiers['strucHard'] > 1 || w.modifiers['strucBunker'] > 1) {
					// log('--to bomber');
					pushByTime(wpnCats, wpn, 'bomber');
					break;
				}
				// log('--to ground support');
				pushByTime(wpnCats, wpn, 'gunship');
				break;
			}
			case unitTypes.Cyborg: {
				// log('--to borg');
				if (w.modifiers['strucMedium'] >= 1 || w.modifiers['strucHard'] >= 1 || w.modifiers['strucBunker'] >= 1) {
					// log('--to arty');
					pushByTime(wpnCats, wpn, 'borg_strc');
				}
				if (w.modifiers['Cyborg Propulsion'] >= 1) {
					// log('--to raider');
					pushByTime(wpnCats, wpn, 'borg_pers');
				}
				// log('--to main');
				if (w.modifiers['Wheels'] + w.modifiers['Half-tracks'] + w.modifiers['Tracks'] + w.modifiers['Hover'] >= 4) {
					pushByTime(wpnCats, wpn, 'borg_tank');
				}
				break;
			}
		}

		if (componentAvailable(w.id)) {
			eventResearched({ id: w.tech }, undefined, me);
		}
	}

}